<?php

namespace App\Http\Controllers\API;

use App\Models\Company;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Traits\AuctionTrait;
use App\Mail\Auction\BidMail;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

class AuctionController extends Controller
{
    use AuctionTrait;

    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function index(Request $request)
    {
       $user= $request->user();
       $auctions = $this->getActiveAuctions($user->Loyalty_programme_id);
       return response()->json([
           "status"=>1,
           "status_code"=>200,
           "message"=>"success",
           "auctions"=>$auctions
       ]);
    }

    public function detail(Request $request,$auctionId)
    {
       $user= $request->user();
       $auction = $this->getAuction($user->Loyalty_programme_id,$auctionId);

       if(count($auction) < 1)
       {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"Auction does not exist"
            ]);
        }
        $still_active =$auction->first();
        return response()->json([
            "status"=>1,
            "status"=>200,
            "message"=>"success",
            "auction"=>$still_active
        ]);
    }

    public function submitBid(Request $request,$auctionId)
    {
        $user= $request->user();
        $auction = $this->getAuction($user->Loyalty_programme_id,$auctionId);
        if(count($auction) <1){
             return response()->json([
                 "status"=>0,
                 "status_code"=>404,
                 "message"=>"Auction does not exist"
             ]);
         }
        $active_auction =$this->isAuctionStillActive($auction);
        if(!$active_auction){
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"The bidding time for this auction has elasped"
            ]);
        }

        $user_bid = $request->user_bid;
        $current_bid = $active_auction->Current_bid?$active_auction->Current_bid:$active_auction->Min_bid_value;
        $minimum_bid = $current_bid + $active_auction->Min_increment;

        if($user_bid > ($user->Current_balance - $user->Blocked_points))
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"You dont have sufficient fund to bid"
            ]);
        }

        if($user_bid < $minimum_bid)
        {
            return response()->json([
                "status"=>2,
                "current_bid"=>$current_bid,
                "minimum"=>$minimum_bid,
                "status_code"=>400,
                "message"=>"The minimum bid must be ".number_format($minimum_bid)
            ]);
        }
        $company  = Company::with(['mail_configuration'])->where('Company_id',$user->Company_id)->first();
        if($active_auction->Current_bid)
        {
            $this->unBlockedBidderPoint($active_auction,$company,$active_auction->Current_bid);
        }

        DB::table("lsl_auction_winner")->insert([
            'Auction_id'=>$active_auction->Auction_id,
            'Company_id'=>$user->Company_id,
            'Enrollment_id'=>$user->Enrollment_id,
            'Prize'=>$active_auction->Prize,
            'Bid_value'=>$user_bid,
            'Winner_flag'=>0,
            'Active_flag'=>0,
            'Create_user_id'=>29,
            'Creation_date'=>date('Y-m-d H:i:s')
        ]);

        $this->blockedBidderPoint($active_auction,$company,$user,$user_bid);

        return response()->json([
            "status"=>1,
            "current_bid"=>$current_bid,
            "minimum"=>$user_bid+$active_auction->Min_increment,
            "status_code"=>200,
            "message"=>"Success,The current bid is now ".number_format($user_bid)
        ]);
    }
}
