<?php

namespace App\Http\Controllers\API;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\MerchandizeCatalogue;
use Carbon\Carbon;
use Illuminate\Support\Facades\Input;
use Validator;
use App\Http\Resources\Catalogues\ItemCollection as ItemCollection;
use App\Http\Resources\Catalogues\ItemResource as ItemResource;

class ProductController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:api');
    }
    public function category_products(Request $request, $categoryId)
    {
        $loyalty_programme_id = $request->user()->Loyalty_programme_id;
        $date = Carbon::now()->toDateString();

        $validator = Validator::make($request->all(),[
            'delivery_method'=>'nullable|numeric',
            'sort'=>'nullable|numeric',
            'sale'=>'nullable|numeric'
        ]);
        if($validator->fails())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "errors"=>$validator->errors()
            ]);
        }

        $limit = Input::get("limit") <= 0 ? 20 :Input::get("limit");

        $catalogue = MerchandizeCatalogue::whereHas('program',function($query) use($loyalty_programme_id){
            $query->where('Loyalty_program_id',$loyalty_programme_id);
        })->where('Merchandize_category_id',$categoryId)
        ->where('Active_flag',1)->whereDate('valid_from','<=',$date)
        ->whereDate('valid_till','>=',$date);
        
        

        $catalogue = $request->sale==1?$catalogue->where('Sale_flag',1):$catalogue;
        $catalogue = $request->delivery_method >= 1 &&  $request->delivery_method <= 3
        ?$catalogue->where('Delivery_method',$request->delivery_method):$catalogue;

        $total_items = $catalogue->count();
        $catalogue = $request->sort == 1 ? $catalogue->orderBy('Billing_price_in_points',"ASC"):$catalogue;
        //dd($catalogue->toSql());
        $catalogue = $catalogue->paginate($limit);

        return response()->json([
            "total_items"=> $total_items,
            "status"=>1,
            "status_code"=>200,
            "items"=> ItemCollection::collection($catalogue)],200);
    }

    public function show_product(Request $request,$productId)
    {
        $loyalty_programme_id = $request->user()->Loyalty_programme_id;
        $catalogue = MerchandizeCatalogue::with('branches')
        ->whereHas('program',function($query) use($loyalty_programme_id){
            $query->where('Loyalty_program_id',$loyalty_programme_id);
        })->where('Company_Merchandise_item_id',$productId)->first();
        // $catalogue->first();
        
        // dd($catalogue->first());
        //dd($catalogue->toSql());
        if($catalogue==null)
        return response()->json([
            "status_code"=>404,
            "status"=>0,
            "message"=>"Product not found"
        ],200);

        return response()->json(new ItemResource($catalogue),200);
    }

    // public function category_products_onsale($categoryId)
    // {
    //     $date = Carbon::now()->toDateString();
    //     $catalogue = MerchandizeCatalogue::where('Merchandize_category_id',$categoryId)
    //     ->where('Active_flag',1)->where('valid_from','<=',$date)
    //     ->where('valid_till','>=',$date)->get();
    //     return response()->json($catalogue,200);
    // }
}
