<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Models\Company;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\LoyaltyProgram;
use App\Models\EnrollmentMaster;
use App\Mail\EnrollmentMail;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

class RegisterController extends Controller
{
    public function registerUser(Request $request,$company_slug)
    {
        $validator = \Validator::make($request->all(),[
            'First_name'=>'required',
            'Last_name'=>'required',
            'User_email_id'=>'required|email',
            'Password'=>'required'
        ]);
        if($validator->fails())
        {
            $response['message']="Invalid request";
            $response["status_code"]=400;
            $response["status"]=0;
            $response['errors']=$validator->errors();
            return response()->json($response);
        }

        $program = LoyaltyProgram::with(['company'])->where([['slug',$company_slug]])->first();
//return response()->json($program->company->mail_configuration->toArray()["SMTP_username"]);
        if($program == null)
        {
            $response['message']="Loyalty program does not exist";
            $response['status']=0;
            $response['status_code']=404;
            return response()->json($response);
        }
        if($program->company->Membership_generation == 0)
        {
            $response['message']="The loyalty program does not allow auto enrollment";
            $response['status']=0;
            $response['status_code']=400;
            return response()->json($response);
        }

        if($program->Total_members_enrolled >= $program->Members_enrollment_limit)
        {
            $response['message']="The Loyalty program has reach the limit of members";
            $response['status']=0;
            $response['status_code']=400;
            return response()->json($response);
        }

        $member_result = $this->getMembershipID($program->Company_id);
        if($member_result == null)
        {
            $response['message']="There is no membership ID to assigned to user";
            $response['status']=0;
            $response['status_code']=400;
            return response()->json($response);
        }

        $db_user = EnrollmentMaster::where("User_email_id",$request->User_email_id)
                ->where('Company_id',$program->Company_id)
                ->where('Loyalty_programme_id',$program->Loyalty_program_id)->first();
        if($db_user != null)
        {
            $response['message']="This email has already been used for this loyalty program";
            $response['status']=0;
            $response['status_code']=400;
            return response()->json($response);
        }

            $user = new EnrollmentMaster();
            $user->fill($request->all());

            $user->Membership_id = $member_result->Membership_id;
            $user->Company_id = $program->Company_id;
            $user->Loyalty_programme_id = $program->Loyalty_program_id;
            $user->pin = rand(1000,9999);
            $user->User_type_id=1;
            $user->Communication_flag=1;
            $user->Tier_id = $program->company->tiers->first()->Tier_id;
            $user->State_id=$program->Company_id;
            $user->Region_id=$program->company->Region_id;
            $user->Zone_id=$program->company->Zone_id;
            $user->City_id =$program->company->City_id;
            $user->Country_id=$program->company->Country_id;
            $user->Enroll_date=date("Y-m-d");


            //Update the Membership ID to Utilized
            \DB::table('lsl_automated_membershipid')->where('Company_id',$member_result->Company_id)
            ->where("Membership_id",$member_result->Membership_id)->update(["Utilized"=>1]);
            $user->save();

            CustomMailJob::dispatch($program->company->mail_configuration->toArray(),$user->User_email_id,
            new EnrollmentMail($program->company,$user));

            //Update the total member enrolled
            $program->Total_members_enrolled =+1;
            $program->save();

            $response['message']="User has been enrolled successfully";
            $response['status']=1;
            $response['status_code']=400;
            $response["member"]=$user;
            return response()->json($response);

    }

    private function getMembershipID($company_id)
    {
       $membership = DB::table('lsl_automated_membershipid')
        ->where('Company_id',$company_id)->where("Utilized",0)->first();
        if($membership == null)
        {
            return null;
        }
        $memberid_count = DB::table('lsl_enrollment_master')->where("Company_id",$company_id)
            ->where('Membership_id',$membership->Membership_id)->count();
        if($memberid_count > 0)
        {
            return null;
        }
        return $membership;
    }
}
