<?php

namespace App\Http\Controllers\API;

use App\Traits\UserNotificationTrait;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Input;

class UserNotificationController extends Controller
{
    use UserNotificationTrait;

    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function index(Request $request)
    {
        $user = $request->user();
        $limit = Input::get('limit');
        $read = Input::get('read')!=null?Input::get('read'):null;

        $notifications = $this->getUserNotifications($user->Membership_id,$limit,$read);
        return response()->json([
            "notifications"=>$notifications,
            "status"=>1,
            "status_code"=>200
        ]);
    }

    public function fullNotifications(Request $request)
    {
        $user = $request->user();
        $limit = Input::get('limit');
        $read = Input::get('read')!=null?Input::get('read'):null;

        $notifications = $this->getUserFullNotifications($user->Membership_id,$limit,$read);
        return response()->json([
            "notifications"=>$notifications,
            "status"=>1,
            "status_code"=>200
        ]);
    }

    public function readNotification(Request $request,$notification_id)
    {
        $user = $request->user();
        $notification = $this->getUserNotificationContent($user->Membership_id,$notification_id);
        if($notification==null)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"User notification is not found"
            ]);
        }
        $this->markAsOpen($notification_id);
        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "message"=>"Notification mark as read"
        ]);
    }

    protected function getNotificationContent(Request $request,$notification_id)
    {
        $user = $request->user();
        $notification = $this->getUserNotificationContent($user->Membership_id,$notification_id);
        if($notification==null)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"User notification is not found"
            ]);
        }
        $this->markAsOpen($notification_id);
        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "Contents"=>$notification->Contents
        ]);
    }

    public function deleteNotification(Request $request,$notification_id)
    {
        $this->deleteUserNotification($notification_id);
        return response()->json((object)[
            'status'=>1,
            "message"=>'You have deleted a notification'
        ]);
    }
}
