<?php

namespace App\Mail\Transfer;

use App\Traits\MailNotification;
use App\Traits\UserNotificationTrait;
use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Contracts\Queue\ShouldQueue;


class CreditMail extends Mailable
{
    use Queueable, SerializesModels,MailNotification,UserNotificationTrait;

    private $recipient;
    private $sender;
    private $amount;
    private $company;

    public function __construct($sender,$user,$amount,$company)
    {
        $this->recipient = $user;
        $this->sender = $sender;
        $this->amount = $amount;
        $this->company = $company;

    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {
        $email_structure = $this->getEmailStructure($this->sender->Company_id,35);
        $email_structure->Email_subject = str_replace('$Tranfered_points',$this->amount,$email_structure->Email_subject);
        $email_variables = $this->getEmailVariable("Customer Transfer To Points");
        $values = $this->assignValues($this->sender,$this->recipient,$this->amount,$this->company);
        $email_structure->Footer_notes =
            str_replace('$Website', $this->company->Website,$email_structure->Footer_notes);
        foreach($email_variables as $variable)
        {
            if(isset($values["".$variable.""]))
                $email_structure->Email_body = str_replace($variable,$values["".$variable.""],$email_structure->Email_body);
        }
        $this->createNotification($this->recipient,$email_structure->Email_body,3,0);

        return $this->from($this->company->Company_person_Email,$this->company->Company_name)
            ->subject($email_structure->Email_subject)->view('mail.general')->with(['structure'=>$email_structure]);
    }

    private function assignValues($sender,$recipient,$amount,$company)
    {
        return ['$First_name'=>$recipient->First_name,'$Last_name'=>$recipient->Last_name,
            '$Tranfered_from_name'=>$sender->First_name." ".$sender->Last_name,'$Gained_points'=>$amount,
            '$Current_Balance'=>$recipient->Current_balance, '$Website'=>$company->Website,'$Application_website'=>$company->Website,
            '$Facebook_icon'=>'','$Twitter_icon'=>'','$Google_plus_icon'=>'','$Linkedin_icon'=>'',' $iOS_icon'=>'',
            '$Android_icon'=>''];
    }
}
