<?php

namespace App\Traits;

use App\Jobs\CustomMailJob;
use DB;
use App\Models\EnrollmentMaster;
use App\Mail\Auction\ReBidMail;
use App\Mail\Auction\BidMail;
use Illuminate\Support\Facades\Mail;

trait AuctionTrait
{
    protected function getActiveAuctions($loyalty_program_id)
    {
        $now = date("Y-m-d",time());
        $time = date("h:i:s",time());
        return DB::table('lsl_auction_master as auc')
        ->leftJoin('lsl_auction_winner as win','win.Auction_id','=','auc.Auction_id')
        ->select(DB::Raw('MAX(win.Bid_value) As Current_bid,CONCAT("https://demo.perxclm.com/",auc.Prize_image) AS Prize_image'),'auc.Auction_id',
        'auc.End_time','auc.To_date','auc.From_date','auc.Auction_name','auc.Prize','auc.Min_bid_value')
        ->groupBy(['auc.End_time','auc.Auction_id','auc.To_date','auc.Min_bid_value','auc.Auction_name','auc.Prize',
        'auc.Prize_image','auc.From_date'])
        ->whereDate('auc.From_date','<=',$now)->where('auc.To_date','>=',$now)->when('auc.To_date'<=$now,function($query)use($time){
            return $query->whereTime('End_time','<=',$time);
        })->where('auc.Loyalty_program_id',$loyalty_program_id)->get();
    }

    protected function getAuction($loyalty_program_id,$auctionId)
    {

        $now = date("Y-m-d");
        return DB::table('lsl_auction_master as auc')
        ->leftJoin('lsl_auction_winner as win','win.Auction_id','=','auc.Auction_id')
        ->select(DB::Raw('MAX(win.Bid_value) As Current_bid,COUNT(win.Auction_id) as Bids,CONCAT("https://demo.perxclm.com/",auc.Prize_image) AS Prize_image'),
        'auc.End_time','auc.Auction_id','auc.To_date','auc.Auction_name','auc.Prize',
        'auc.Min_bid_value','auc.Min_increment','auc.Prize_description','auc.From_date')
        ->groupBy(['auc.End_time','auc.Auction_id','auc.To_date','auc.Auction_name','auc.Prize',
        'auc.Prize_image','auc.Min_bid_value','auc.Min_increment','auc.Prize_description','auc.From_date'])
        ->whereDate('auc.From_date','<=',$now)->where('auc.To_date','>=',$now)
        ->where('auc.Loyalty_program_id',$loyalty_program_id)->where('auc.Auction_id',$auctionId)->get();
    }

    protected function isAuctionStillActive($auction)
    {
       $time = date("h:i:s",time());
       $now = date('Y-m-d',time());
       return $auction->when('auc.To_date'<=$now,function($query)use($time){
            return $query->whereTime('End_time','<=',$time);
        })->first();
    }

    protected function unBlockedBidderPoint($auction,$company,$amount)
    {
        $enrollmentId = DB::table('lsl_auction_winner')->where('Auction_id',$auction->Auction_id)
        ->orderBy('Bid_value','desc')->first()->Enrollment_id;
        $user = EnrollmentMaster::where('Enrollment_id',
                    $enrollmentId)->first();
        $user->Blocked_points+=$amount;
        $user->save();

        if($user)
        {
            try{
                CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                    new ReBidMail($user,$auction,$company,$amount));
            }catch (\Exception $ex)
            {

            }
        }


    }

    protected function blockedBidderPoint($auction,$company,$user,$amount)
    {
        $user = EnrollmentMaster::where('Enrollment_id',
                    $user->Enrollment_id)->first();
        $user->Blocked_points+=$amount;
        $user->save();
        if($user)
        {
            try{
                CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                    new BidMail($user,$auction,$company,$amount));
            }catch (\Exception $exception)
            {

            }
        }


    }
}
