<?php

namespace App\Http\Controllers\API;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\MerchandizeCatalogue;
use App\Models\TempCart;
use App\Traits\CartTrait;
use App\Http\Resources\Cart\CartCollection as CartCollection;
use Illuminate\Support\Facades\Validator;


class CartController extends Controller
{
    use CartTrait;
    public function __construct()
    {
        return $this->middleware('auth:api');
    }

    public function cart_items(Request $request)
    {
        $user = $request->user();
        $cart_items = CartCollection::collection($this->getUserCartItems($user->Enrollment_id));
        return response()->json([
            "cart_total"=>$cart_items->sum("Amount"),
            "count"=>count($cart_items),
            "items"=>$cart_items,
            "status_code"=>200,
            "status"=>1
        ]);
    }

    public function add_to_cart(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
            'remark'=>"nullable"
        ]);

        if($validator->fails()){
            return response()->json([
                "status_code"=>400,
                "status"=>0,
                "message"=>"Invalid user request",
                "errors"=>$validator->errors(),
            ]);
        }
        $buyer = $request->user();
        $item = MerchandizeCatalogue::with('branches')
                ->where('Merchandize_item_code',$request->item_code)->first();
        $cart_query =$this->getUserCart($buyer->Enrollment_id)->where('Item_code',$item->Merchandize_item_code);
        if($request->redemption_method==1)
        {
            if($item->Delivery_method==2)
            {
                return response()->json([
                    "status"=>5,
                    "message"=>"This delivery method is not supported for this product",
                    "status_code"=>400
                ]);
            }
            $branches = $item->branches->pluck("Branch_code")->toArray();
            if(!in_array($request->branch,$branches))
            {
                return response()->json([
                    "status"=>4,
                    "status_code"=>400,
                    "message"=>"Invalid Branch code"
                ]);
            }
            $cart_item = $cart_query->Where('Redemption_method',$request->redemption_method)
                            ->where('Branch',$request->branch)->first();
        }else{

            if($item->Delivery_method==1)
            {
                return response()->json([
                    "status"=>5,
                    "status_code"=>400,
                    "message"=>"This delivery method is not supported for this product"
                ]);
            }
           $cart_item = $cart_query->Where('Redemption_method',$request->redemption_method)
           ->first();
        }
        if($cart_item == null)
        {
            $buyer_cart_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
            if($buyer->walletBalance() < ($item->Billing_price_in_points + $buyer_cart_total))
            {
                return response()->json([
                    "status_code"=>400,
                    "status"=>2,
                    "current_balance"=>$buyer->Current_balance-$buyer_cart_total,
                    "item_price"=>$item->Billing_price_in_points,
                    "message"=>"Insufficient balance to purchase this product"
                ]);
            }
            $item_add = TempCart::create([
                'Item_code'=>$item->Merchandize_item_code,
                'Enrollment_id'=>$buyer->Enrollment_id,
                'Remark'=>$request->remark,
                'Branch'=>$request->branch,
                'Redemption_method'=>$request->redemption_method,
                'Company_id'=>$buyer->Company_id,
            ]);
            if($item_add != null)
            {
                $new_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
                return response()->json([
                    "status"=>1,
                    "status_code"=>200,
                    "cart_total"=>$new_total,
                    "message"=>"Item has been Added to cart"
                ]);
            }
        }
        return response()->json([
            "status"=>3,
            "message"=>"Item already Exist in cart",
            "status_code"=>400
        ]);
    }


    public function update_cart(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
            'quantity'=>'required|numeric|min:1'
        ]);
        if($validator->fails()){
            return response()->json([
                "status_code"=>400,
                "status"=>0,
                "message"=>"Invalid user request",
                "errors"=>$validator->errors()
            ]);
        }
        $buyer = $request->user();

        $cart_items = $this->getUserCart($buyer->Enrollment_id);
        if(count($cart_items) <= 0)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"There is no Item in the Cart"
            ]);
        }

        $cart_items = $cart_items->where('Item_code',$request->item_code)
        ->where('Branch',$request->branch)
            ->Where('Redemption_method',$request->redemption_method);

        $quantity_in_cart = count($cart_items);
        if($quantity_in_cart <= 0)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"Item not in cart"
            ]);
        }

        if($quantity_in_cart < $request->quantity)
        {
            $cart_item = $this->getSingleCartItem($cart_items->first()->Temp_cart_id);

            $cart_totals = $this->getUserCart($buyer->Enrollment_id)->where("Item_code","<>",
                            $cart_item->Item_code)->sum('product.Billing_price_in_points');

            $required_amount =  ($cart_item->product->Billing_price_in_points * $request->quantity)
                                + $cart_totals;

            if($buyer->Current_balance < $required_amount)
            {
                $response["message"]="You don't have sufficient funds for this action";
                $response["status"]= 0;
                $response["status"]=400;
                $response['item_code']=$cart_item->Item_code;
                $response['redemptionn_method']=$cart_item->Redemption_method;
                $response['branch']= $cart_item->Branch;
                $response['quantity'] = $quantity_in_cart;
                $response['amount_required']=$required_amount;
                $response['current_balance']=$buyer->Current_balance;
                return response()->json($response);
            }

            for($i=1; $i<=($request->quantity - $quantity_in_cart); $i++){

                TempCart::create([
                    'Item_code'=>$cart_item->Item_code,
                    'Enrollment_id'=>$buyer->Enrollment_id,
                    'Remark'=>$request->remark,
                    'Branch'=>$request->branch,
                    'Redemption_method'=>$request->redemption_method,
                    'Company_id'=>$buyer->Company_id,
                ]);
            }
            $response["message"] ="Item has been updated successully";
            $response["status"] = 1;
            $response["status_code"]=200;
            $response["cart_total"] = $required_amount;
            return response()->json($response);
        }else{

            $count = $quantity_in_cart - $request->quantity;
            foreach($cart_items as $temp_item)
            {
                if($count <= 0)
                {
                    $new_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
                    $response["message"]="Item has been updated successully";
                    $response["status"]= 1;
                    $response["status_code"]=200;
                    $response["cart_total"]=$new_total;
                    return response()->json($response);
                }
                $temp_item->delete();
                $count--;
            }
        }
    }


    public function delete_item(Request $request)
    {
        $buyer = $request->user();
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
        ]);
        if($validator->fails())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"Invalid user request",
                "errors"=>$validator->errors()
            ]);
        }
        $carts = $this->getUserCart($buyer->Enrollment_id);
        if(count($carts) > 0)
        {
            $carts->where('Item_code',$request->item_code)->where('Branch',$request->branch)
            ->Where('Redemption_method',$request->redemption_method)->delete();
            $cart_totals =$this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
            $response['status']= 1;
            $response["status_code"]=200;
            $response['message']="Item Removed Successfully";
            $response['cart_total']=$cart_totals;
            return response()->json($response);
        }
        return response()->json([
            "status"=>0,
            "status_code"=>400,
            "message"=>"No cart item found"
        ]);
    }
}
