<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Mail\Passwords\RequestResetMail;
use App\Mail\Passwords\ResetPasswordMail;
use App\Mail\RedemptionMail;
use App\Models\Company;
use App\Models\Passwords\PasswordReset;
use App\Traits\PasswordTrait;
use Carbon\Carbon;
use Illuminate\Foundation\Auth\SendsPasswordResetEmails;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class PasswordController extends Controller
{
    use PasswordTrait;


    public function requestPasswordReset(Request $request,$slug)
    {
        $validator =  Validator::make($request->all(),[
           'email'=>'required',
            'password_callbackurl'=>'required'
        ]);

        if($validator->fails())
        {
          $response['status']=0;
          $response["status_code"]=400;
          $response['errors']=$validator->errors();
          return response()->json($response);
        }
        $customer  = $this->getMembersWithEmail($request->email,$slug);
        if($customer == null)
        {
            $response['status']=0;
            $response["status_code"]=400;
            $response['message']="Invalid User email ID";
            return response()->json($response);
        }
        $token = hash('sha256', Str::random(40));

        $callback_url = $request->password_callbackurl."?token=".$token;
        $passwordReset = PasswordReset::updateOrCreate(
            ['User_email_id' => $customer->User_email_id,
            ],
            [
                'User_email_id' => $customer->User_email_id,
                'Token' => $token,
                'Expiry_time'=>Carbon::now()->addHours(24)->toDateTime(),
                'Membership_id'=>$customer->Membership_id
            ]
        );
        $company = Company::with('mail_configuration')->where('Company_id',$customer->Company_id)->first();
        if($customer->mail_configuration && $passwordReset)
        {
            try{
                CustomMailJob::dispatch($company->mail_configuration->toArray(),$customer->User_emal_id,
                    new RequestResetMail($customer,$callback_url,$company))->onQueue("emails");
                return response()->json([
                    'status'=>1,
                    "status_code"=>200,
                    'message' => 'We have e-mailed your password reset link!'
                ]);
            }catch (\Exception $ex)
            {
                return response()->json([
                    "status"=>0,
                    "status_code"=>400,
                    "message"=>"Could not send password reset link",
                    "reason"=>$ex->getMessage()
                ]);
            }
        }

        return response()->json([
            "status"=>0,
            "status_code"=>400,
            "message"=>"Couldn't find the mail configurations for the company"
        ]);
    }

    public function verifyToken(Request $request)
    {
        $passwordReset = PasswordReset::where([
                ['token', $request->token]
        ])->first();
        if (!$passwordReset)
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message' => 'This password reset token is invalid.'
            ]);
        if (Carbon::parse($passwordReset->updated_at)->addHours(24)->isPast()) {
            $passwordReset->delete();
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message' => 'This password reset token is invalid.'
            ]);
        }
        return response()->json([
            'status'=>1,
            "status_code"=>200,
            'message'=>'Token is valid, please enter your password',
            'Reset_token'=>$passwordReset
        ]);
    }

    public function reset(Request $request,$slug)
    {
        $validator = Validator::make($request->all(),[
            'email' => 'required|string|email',
            'password' => 'required|string',
            'token' => 'required'
        ]);
        if($validator->fails())
        {
            $response['status']=0;
            $response["status_code"]=400;
            $response['message']="Invalid user request";
            $response['errors']=$validator->errors();
            return response()->json($response);
        }

        $passwordReset = PasswordReset::where('Token', $request->token)
                                    ->where('User_email_id', $request->email)->first();

        if($passwordReset==null)
        {
            return response()->json([
                'message' => 'This password reset token is invalid.',
                "status_code"=>400,
                'status'=>0
            ]);
        }

        $customer =$this->getMembersWithEmail($request->email,$slug);
        if ($customer==null)
        {
            return response()->json([
                'message' => 'We can not find a user with that e-mail address.',
                'status'=>0,
                "status_code"=>400,
            ]);
        }
        $company = Company::with('mail_configuration')->find($customer->Company_id);

        $customer->Password = $request->password;
        $customer->save();
        $passwordReset->delete();

        if($company->mail_configuration==null)
        {
            return response()->json([
                'status'=>1,
                "status_code"=>200,
                'message'=>'You password has been changed but could not send success mail',
               ]);
        }

        try{
            CustomMailJob::dispatch($company->mail_configuration->toArray(),$customer->User_email_id,
                new ResetPasswordMail($customer,$company))->onQueue("emails");

            return response()->json([
                'status'=>1,
                "status_code"=>200,
                'message'=>'You password has been changed',
                'customer'=>$customer]);
        }catch (\Exception $exception)
        {
            return response()->json([
                'status'=>1,
                "status_code"=>200,
                'message'=>'You password has been changed but could not send success mail',
                'reason'=>$exception->getMessage()]);
        }
    }


    public function changePassword(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'current_password'=>'required',
            'new_password'=>'required'
        ]);
        if($validator->fails())
        {
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message'=>'Invalid request',
                'errors'=>$validator->errors()
            ]);
        }

        $customer = $request->user();
        if($customer->Password !==$request->current_password)
        {
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message'=>'Incorrect password'
            ]);
        }
        $customer->Password = $request->new_password;
        $customer->save();
        return response()->json([
            'status'=>1,
            "status_code"=>200,
            'message'=>'Password changed successfully'
        ]);
    }

}
