<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Mail\Passwords\RequestResetMail;
use App\Mail\Passwords\ResetPasswordMail;
use App\Models\Company;
use App\Models\Passwords\PasswordReset;
use App\Traits\RedeemPinTrait;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class PinController extends Controller
{
    public function __construct()
    {
        $this->middleware("auth:api");
    }
    use RedeemPinTrait;
    public function pinResetRequest(Request $request)
    {
        $validator =  Validator::make($request->all(),[
            'pin_callbackurl'=>'required'
        ]);
        if($validator->fails())
        {
            $response['status']=0;
            $response["status"]=400;
            $response['message']=$validator->errors()->first('pin_callbackurl');
            return response()->json($response);
        }
        $customer  = $request->user();

        $result = $this->sendPinResetRequest($customer,$request->pin_callbackurl);
        if($result)
        {
            return response()->json([
                'status'=>1,
                 "status_code"=>200,
                'message' => 'We have e-mailed you a pin reset link!'
            ]);
        }
        return response()->json([
            'status'=>0,
            "status_code"=>400,
            'message' => 'Oops!, an error occurred, please try again'
        ]);
    }
    public function verifyToken(Request $request)
    {
        $user = $request->user();
        $pin_reset = $this->checkToken($user,$request->token);
        if (!$pin_reset)
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message' => 'This pin reset token is invalid.'
            ]);
        if(Carbon::parse($pin_reset->updated_at)->addHours(5)->isPast())
        {
            $pin_reset->delete();
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message' => 'This pin reset token has expired.'
            ]);
        }
        return response()->json([
            'status'=>1,
            "status_code"=>200,
            'message'=>'Token is valid, please enter your new pin',
            'reset_pin'=>$pin_reset
        ]);
    }

    public function resetPin(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'pin' => 'required|max:4|min:4',
            'token' => 'required'
        ]);
        if($validator->fails())
        {
            $response['status']=0;
            $response["status_code"]=400;
            $response['message']="Invalid request";
            $response['errors']=$validator->errors();
            return response()->json($response);
        }

        $user = $request->user();
        $pin_reset = $this->checkToken($user,$request->token);

        if($pin_reset==null)
        {
            return response()->json([
                'message' => 'This pin reset token is invalid.',
                'status'=>0,
                "status_code"=>400,
            ]);
        }

        $user->Pin = $request->pin;
        $user->save();
        $this->clearRedeemAttempt($user);
        $this->sendPinResetSuccess($user);
        return response()->json([
            'status'=>1,
            "status_code"=>200,
            'mesasage'=>'You pin has been changed'
        ]);
    }

    public function changePin(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'current_pin'=>'required',
            'new_pin'=>'required'
        ]);
        if($validator->fails())
        {
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message'=>'Invalid request',
                'errors'=>$validator->errors()
            ]);
        }
        $customer = $request->user();
        if($customer->Pin !==$request->current_pin)
        {
            return response()->json([
                'status'=>0,
                "status_code"=>400,
                'message'=>'The pin you entered is incorrect'
            ]);
        }
        $customer->Pin = $request->new_pin;
        $customer->save();
        return response()->json([
            'status'=>1,
            "status_code"=>200,
            'message'=>'Pin has been changed successfully'
        ]);
    }

}
