<?php

namespace App\Http\Controllers\API;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\MerchandizeCatalogue;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Input;
use Illuminate\Support\Facades\Validator;
use App\Http\Resources\Catalogues\ItemCollection as ItemCollection;
use App\Http\Resources\Catalogues\ItemResource as ItemResource;

class ProductController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function category_products(Request $request, $categoryId)
    {
        $loyalty_programme_id = $request->user()->Loyalty_programme_id;
        $date = Carbon::now()->toDateString();

        $validator = Validator::make($request->all(),[
            'delivery_method'=>'nullable|numeric',
            'sort'=>'nullable|numeric',
            'sale'=>'nullable|numeric'
        ]);
        if($validator->fails())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "errors"=>$validator->errors()
            ]);
        }

        $limit = Input::get("limit") <= 0 ? 20 :Input::get("limit");

        $catalogue = MerchandizeCatalogue::whereHas('program',function($query) use($loyalty_programme_id){
            $query->where('Loyalty_program_id',$loyalty_programme_id);
        })->where('Merchandize_category_id',$categoryId)
        ->where('Active_flag',1)->whereDate('valid_from','<=',$date)
        ->whereDate('valid_till','>=',$date);

        $catalogue = $request->sale==1?$catalogue->where('Sale_flag',1):$catalogue;
        $catalogue = $request->delivery_method >= 1 &&  $request->delivery_method <= 3
        ?$catalogue->where('Delivery_method',$request->delivery_method):$catalogue;

        $total_items = $catalogue->count();
        $catalogue = $request->sort == 1 ? $catalogue->orderBy('Billing_price_in_points',"ASC"):$catalogue;
        $catalogue = $catalogue->paginate($limit);

        return response()->json([
            "total_items"=> $total_items,
            "status"=>1,
            "status_code"=>200,
            "items"=> ItemCollection::collection($catalogue)],200);
    }

    public function show_product(Request $request,$productId)
    {
        $loyalty_programme_id = $request->user()->Loyalty_programme_id;
        $catalogue = MerchandizeCatalogue::with('branches')
        ->whereHas('program',function($query) use($loyalty_programme_id){
            $query->where('Loyalty_program_id',$loyalty_programme_id);
        })->where('Company_Merchandize_item_code',$productId)->first();

        if($catalogue==null)
        {
            return response()->json([
                "status_code"=>404,
                "status"=>0,
                "message"=>"Product not found"
            ],200);
        }

        return response()->json(new ItemResource($catalogue),200);
    }

    public function mostRedeemItems(Request $request)
    {
        $programme_id = $request->user()->Loyalty_programme_id;
        $most_redeemed_items = DB::table("lsl_redeemption_contactdetails AS lrc")
            ->join("lsl_company_merchandise_catalogue AS lcml",function ($query)use($programme_id){
                $query->on("lcml.Merchandize_item_code","=","lrc.Item_code")
                    ->where(["lcml.Loyalty_program_id"=>$programme_id,"lcml.Sale_flag"=>1,
                    "lcml.Active_flag"=>1]);
            })->groupBy(["lcml.Merchandize_item_name","Item_image","lcml.Merchandise_item_description",
                "Item_code","lcml.Cost_in_points"])
            ->select("lcml.Merchandize_item_name","Item_image","lcml.Merchandise_item_description",
                "Item_code","lcml.Cost_in_points AS Price",DB::raw("COUNT(Item_code) AS Redemption"))
//            ->where("Status_flag",">",0)
            ->orderByDesc("Redemption")->take(10)->get();

        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "most_redeemed"=>$most_redeemed_items]
            ,200);
    }
}
