<?php

namespace App\Http\Controllers\API;

use App\Models\WishList;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class WishlistController extends Controller
{
    public function __construct()
    {
        $this->middleware("auth:api");
    }

    public function userWishLists(Request $request)
    {
        $enrollment_id = $request->user()->Enrollment_id;
        $wish_lists = DB::table("lsl_wish_lists AS wish")
                        ->join("lsl_company_merchandise_catalogue AS cat",
                            "cat.Merchandize_item_code","=","wish.Merchandise_item_code")
                        ->where(["wish.Active_flag"=>1,"wish.Enrollment_id"=>$enrollment_id])
                        ->groupBy(["cat.Merchandize_item_code","wish.Wish_list_id","cat.Merchandize_item_name",
                            "cat.Billing_price_in_points","cat.Item_image","cat.Delivery_method"])
                        ->select("cat.Merchandize_item_code AS Item_code","wish.Wish_list_id","cat.Merchandize_item_name AS Item_name",
                            "cat.Billing_price_in_points AS Item_price","cat.Item_image","cat.Delivery_method")->get();
        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "message"=>"Request successful",
            "wish_lists"=>$wish_lists
        ]);
    }

    public function addWishListItem(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'item_code'=>"required"
        ]);
        if($validator->fails())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"Invalid user request",
                "errors"=>$validator->errors()
            ]);
        }
        $user = $request->user();

        $item = DB::table("lsl_company_merchandise_catalogue")->where([
            "Loyalty_program_id"=>$user->Loyalty_programme_id])->first();
        if($item==null)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"Item not found in catalogue",
            ]);
        }
        $wish_lists = DB::table("lsl_wish_lists")->where("Enrollment_id",$user->Enrollment_id)->get();
        if($wish_lists->where("Merchandise_item_code",$item->Merchandize_item_code)->isNotEmpty())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"Item already exist on your wish lists",
            ]);
        }

        if($wish_lists->count() >20)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"You can't have more than 20 items on your wish list",
            ]);
        }

        DB::table("lsl_wish_lists")->insert([
            "Enrollment_id"=>$user->Enrollment_id,
            "Merchandise_item_code"=>$item->Merchandize_item_code,
            "Created_at"=>Carbon::now()
        ]);
        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "message"=>"Item has been added to wish list successfully",
        ]);
    }

    public function deleteWishListItemItem(Request $request)
    {
        $validator = Validator::make($request->all(),[
           "wish_list_id"=>"required|numeric"
        ]);
        if($validator->fails())
        {
            return response()->json([
                "status"=>0,
                "status_code"=>400,
                "message"=>"Invalid user request",
                "errors"=>$validator->errors()
            ]);
        }
        $enrollment_id = $request->user()->Enrollment_id;

        $wish_list = WishList::where(["Wish_list_id"=>$request->wish_list_id,
                                "Enrollment_id"=>$enrollment_id])->first();
        if($wish_list==null)
        {
            return response()->json([
                "status"=>0,
                "status_code"=>404,
                "message"=>"Wish list item not found",
            ]);
        }

        $wish_list->delete();
        return response()->json([
            "status"=>1,
            "status_code"=>200,
            "message"=>"Wish list item has been removed",
        ]);
    }


}
