<?php

namespace App\Mail\Passwords;

use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Contracts\Queue\ShouldQueue;

class RequestResetMail extends Mailable
{
    use Queueable, SerializesModels;
    private $callback_url;
    private $user;
    public $company;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($user,$callback_url,$company)
    {

        $this->callback_url = $callback_url;
        $this->user = $user;
        $this->company = $company;
    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {
        return $this->subject('Password Reset Request Mail')
            ->markdown('vendor.notifications.email')->with([
                "level" => "default",
                "greeting" => "Hi, ".$this->user->First_name."!",
                "introLines" => [
                    "You are receiving this email because we 
                    received a password reset request for your account."
                ],
                "actionText" => "Reset Password",
                "company" => $this->company,
                "actionUrl" => $this->callback_url,
                "outroLines" => [
                    "If you did not request a password reset, 
                            no further action is required."
                ]
            ])->with([ "company" => $this->company]);
    }
}
