<?php

namespace App\Mail;

use App\Traits\UserNotificationTrait;
use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Traits\MailNotification;
use Illuminate\Support\Facades\Mail;

class RedemptionMail extends Mailable
{
    use SerializesModels,MailNotification,UserNotificationTrait;

    public $user="";
    public $company="";
    public $orders;
    public $order_no;
    public $delivery_address;
    public $vouchers;

    public function __construct($company,$recipient,$orders,$order_no,$delivery_address,$vouchers=[])
    {
        $this->company = $company;
        $this->user = $recipient;
        $this->orders = $orders;
        $this->order_no = $order_no;
        $this->delivery_address = $delivery_address;
        $this->vouchers = $vouchers;
    }


    public function build()
    {

        $email_structure = $this->getEmailStructure($this->company->Company_id,13);

        $table = $this->setOrderTables($this->orders,$this->order_no,$this->vouchers,$this->delivery_address);
        $email_variables = $this->getEmailVariable("Redemption");

        $values = $this->assignValues($this->user,$this->company,$table,$this->order_no);
        $email_structure->Email_subject = str_replace('$First_name',$this->user->First_name,$email_structure->Email_subject);

        foreach($email_variables as $variable)
        {
            if(isset($values["".$variable.""]))
            $email_structure->Email_body = str_replace($variable,$values["".$variable.""],$email_structure->Email_body);
        }
        $email_structure->Email_body = str_replace('table',$table,$email_structure->Email_body);
        $email_structure->Footer_notes = str_replace('$Website',$this->company->Website,$email_structure->Footer_notes);
        $this->createNotification($this->user,$email_structure->Email_body,2,$this->order_no);
        return $this->from($this->company->Company_person_Email,$this->company->Company_name)
            ->to($this->user->User_email_id)
        ->subject($email_structure->Email_subject)->view('mail.redemption')->with(['structure'=>$email_structure]);
    }

    private function assignValues($user,$company,$table,$order_no)
    {
        return ['$First_name'=>$user->First_name,'$Last_name'=>$user->Last_name,'$Order_no'=>$order_no,
        '$Website'=>$company->Website,'$Facebook_icon'=>'','$Twitter_icon'=>'','$Google_plus_icon'=>'','$Linkedin_icon'=>'',' $iOS_icon'=>'',
         '$Android_icon'=>'','$Joining_bonus'=>0,'table'=>$table
        ];
    }

    private function setOrderTables($orders,$order_no,$vouchers,$delivery_address)
    {
       $table="<table border='1' width='100%' cellspacing='0' cellpadding='10'>
        <thead>
                <tr>
                <th>
                    Item Name
                </th> 
                <th>
                    Quantity
                </th>
                <th>
                    Price per Item
                </th>
                <th>
                    Redemption Method
                </th>
                <th>
                    Voucher 
                </th>     
            </tr>
           
        </thead>
    <tbody>";

    foreach($orders as $key=>$order)
    {
       $method = $order->Redemption_method==1?'Pick up':'Delivery';
       $voucher_no = $order->Redemption_method == 1?null:@$vouchers[$order->Item_code];
        $table .= "
        <tr> 
        <td rowspan='4'>
            {$order->Item_name}
        </td>
        <td colspan='4' class='table-col'>
            <tr>
                <td>
                {$order->Quantity}
                </td>
                <td>
                {$order->Price}
                </td>
                <td style='text-align: right;'>
                {$method}
                </td>
                <td style='word-wrap:break-word;'>{$voucher_no}</td>
            </tr>
            <tr>   
                <th colspan='2'>
               <strong> Partner Address</strong>
                </th>
                <th colspan='2'>
                <strong>Remark</strong>
                </th>  
            </tr>
            <tr>   
                <td colspan='2'>
                {$order->Partner_address}
                </td>
                <td colspan='2'>
                {$order->Remark}
                </td>  
            </tr>
        </td>

    </tr>
          ";
    }

   $table .="</tbody><tfoot >
        <tr>
            <td colspan='8'><strong>Delivery Address</strong> <br> {$delivery_address}</td>
        </tr>
            </tfoot>
        </table>";
        return $table;
    }
}
