<?php

namespace App\Http\Controllers\API;

use App\Models\Company;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Traits\AuctionTrait;
use App\Mail\Auction\BidMail;
use Illuminate\Support\Facades\Mail;
use DB;

class AuctionController extends Controller
{
    use AuctionTrait;

    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function index(Request $request)
    {
       $user= $request->user();
       $auctions = $this->getActiveAuctions($user->Loyalty_programme_id);
       $response['status'] = 1;
       $response['auctions']=$auctions;
       return response()->json($response);
    }

    public function detail(Request $request,$auctionId)
    {
       $user= $request->user();
       $auction = $this->getAuction($user->Loyalty_programme_id,$auctionId);

       if(count($auction) < 1)
       {
            $response['status'] = 0;
            $response['message']="Auction does not exist";
            return response()->json($response);
        }
        $still_active =$auction->first();
        $response['status'] = 1;
        $response['auction']=$still_active;
        return response()->json($response);
    }

    public function submitBid(Request $request,$auctionId)
    {
        $user= $request->user();
        $auction = $this->getAuction($user->Loyalty_programme_id,$auctionId);
        if(count($auction) <1){

         $response['status'] = 0;
         $response['message']="Auction does not exist";
         return response()->json($response);
         }
        $active_auction =$this->isAuctionStillActive($auction);
        if(!$active_auction){
             $response['status'] = 0;
             $response['message']="The bidding time for this auction has elasped";
             return response()->json($response);
        }

        $user_bid = $request->user_bid;
        $current_bid = $active_auction->Current_bid?$active_auction->Current_bid:$active_auction->Min_bid_value;
        $minimum_bid = $current_bid + $active_auction->Min_increment;

        if($user_bid > ($user->Current_balance - $user->Blocked_points))
        {
            $response['status'] = 0;
            $response['message']="You dont have sufficient fund to bid";
            return response()->json($response);
        }

        if($user_bid < $minimum_bid)
        {
            $response['status'] = 2;
            $response['current_bid']=$current_bid;
            $response['minimum_bid']=$minimum_bid;
            $response['message']="The minimum bid must be ".number_format($minimum_bid);
            return response()->json($response);
        }
        $company  = Company::with(['mail_configuration'])->where('Company_id',$user->Company_id)->first();
        if($active_auction->Current_bid)
        {
            $this->unBlockedBidderPoint($active_auction,$company,$active_auction->Current_bid);
        }

        \DB::table("lsl_auction_winner")->insert([
            'Auction_id'=>$active_auction->Auction_id,
            'Company_id'=>$user->Company_id,
            'Enrollment_id'=>$user->Enrollment_id,
            'Prize'=>$active_auction->Prize,
            'Bid_value'=>$user_bid,
            'Winner_flag'=>0,
            'Active_flag'=>0,
            'Create_user_id'=>29,
            'Creation_date'=>date('Y-m-d H:i:s')
        ]);

        $this->blockedBidderPoint($active_auction,$company,$user,$user_bid);

        $response['status'] = 1;
        $response['message']="The current bid is now ".number_format($user_bid);
        $response['current_bid']=$user_bid;
        $response['minimum_bid']=$user_bid+$active_auction->Min_increment;
        return response()->json($response);
    }
}
