<?php

namespace App\Http\Controllers\API;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\MerchandizeCatalogue;
use DB;
use App\Models\TempCart;
use App\Traits\CartTrait;
use App\Http\Resources\Cart\CartCollection as CartCollection;


class CartController extends Controller
{
    use CartTrait;
    public function __construct()
    {
        return $this->middleware('auth:api');
    }

    public function cart_items(Request $request)
    {
        $user = $request->user();
        $cart_items = CartCollection::collection($this->getUserCartItems($user->Enrollment_id));

            $response["cart_total"] = $cart_items->sum("Amount");
            $response['count'] = count($cart_items);
            $response['Items'] =$cart_items;
            return response()->json($response,200);
    }

    public function add_to_cart(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
            'remark'=>"nullable"
        ]);

        if($validator->fails()){
            $response["message"]="Invalid request";
            $response["errors"]= $validator->errors();
            return response()->json($response);
        }
        $buyer = $request->user();
        $item = MerchandizeCatalogue::with('branches')
                ->where('Merchandize_item_code',$request->item_code)->first();

        $cart_query =$this->getUserCart($buyer->Enrollment_id)->where('Item_code',$item->Merchandize_item_code);
        if($request->redemption_method==1)
        {
            if($item->Delivery_method==2)
            {
                $response['status']= 5;
                $response['message']="This delivery method is not supported for this product";
                return response()->json($response);
            }
            $branches = $item->branches->pluck("Branch_code")->toArray();

            if(!in_array($request->branch,$branches))
            {
                $response['status']= 4;
                $response['message']="Invalid Branch code";
                return response()->json($response);
            }

            $cart_item = $cart_query->Where('Redemption_method',$request->redemption_method)
                        ->where('Branch',$request->branch)->first();
        }else{

            if($item->Delivery_method==1)
            {
                $response['status']= 5;
                $response['message']="This delivery method is not supported for this product";
                return response()->json($response);
            }
           $cart_item = $cart_query->Where('Redemption_method',$request->redemption_method)
           ->first();
        }
        if($cart_item == null)
        {

            $buyer_cart_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
            if($buyer->Current_balance < ($item->Billing_price_in_points + $buyer_cart_total)){
                $response['Current_balance']=$buyer->Current_balance-$buyer_cart_total;
                $response["item_price"]=$item->Billing_price_in_points;
                $response['status']= 2;
                $response['message']="Insufficient balance to purchase this product";
                return response()->json($response);
            }
            $item_add = TempCart::create([
                'Item_code'=>$item->Merchandize_item_code,
                'Enrollment_id'=>$buyer->Enrollment_id,
                'Remark'=>$request->remark,
                'Branch'=>$request->branch,
                'Redemption_method'=>$request->redemption_method,
                'Company_id'=>$buyer->Company_id,
            ]);
            if($item_add != null)
            {
                $new_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
                $response['status']= 1;
                $response["cart_total"]=$new_total;
                $response['message']="Item has been Added to cart";
                return response()->json($response,200);
            }

        }
        $response['status']= 3;
        $response['message']="Item already Exist in cart";
        return response()->json($response,200);
    }


    public function update_cart(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
            'quantity'=>'required|numeric|min:1'
        ]);
        if($validator->fails()){
            $response["message"]="Invalid request";
            $response["errors"]= $validator->errors();
            return response()->json($response);
        }
        $buyer = $request->user();

        $cart_items = $this->getUserCart($buyer->Enrollment_id)->where('Item_code',$request->item_code)
        ->where('Branch',$request->branch)->Where('Redemption_method',$request->redemption_method)->get();

        $quantity_in_cart = count($cart_items);
        if($quantity_in_cart <= 0)
        {
            $response["message"]="Item not in cart";
            $response["status"]= 0;
            return response()->json($response);
        }

        if($quantity_in_cart < $request->quantity)
        {
            $cart_item = $this->getSingleCartItem($cart_items->first()->Temp_cart_id);

            $cart_totals = $this->getUserCart($buyer->Enrollment_id)->where("Item_code","<>",
                            $cart_item->Item_code)->sum('product.Billing_price_in_points');

            $required_amount =  ($cart_item->product->Billing_price_in_points * $request->quantity)
                                + $cart_totals;

            if($buyer->Current_balance < $required_amount)
            {
                $response["message"]="You don't have sufficient funds for this action";
                $response["status"]= 0;
                $response['item_code']=$cart_item->Item_code;
                $response['redemptionn_method']=$cart_item->Redemption_method;
                $response['branch']= $cart_item->Branch;
                $response['quantity'] = $quantity_in_cart;
                $response['amount_required']=$required_amount;
                $response['current_balance']=$buyer->Current_balance;
                return response()->json($response);
            }

            for($i=1; $i<=($request->quantity - $quantity_in_cart); $i++){

                TempCart::create([
                    'Item_code'=>$cart_item->Item_code,
                    'Enrollment_id'=>$buyer->Enrollment_id,
                    'Remark'=>$request->remark,
                    'Branch'=>$request->branch,
                    'Redemption_method'=>$request->redemption_method,
                    'Company_id'=>$buyer->Company_id,
                ]);
            }
            $response["message"] ="Item has been updated successully";
            $response["status"] = 1;
            $response["cart_total"] = $required_amount;
            return response()->json($response,200);
        }else{

            $count = $quantity_in_cart - $request->quantity;
            foreach($cart_items as $temp_item)
            {
                if($count <= 0)
                {
                    $new_total = $this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
                    $response["message"]="Item has been updated successully";
                    $response["status"]= 1;
                    $response["cart_total"]=$new_total;
                    return response()->json($response,200);
                }
                $temp_item->delete();
                $count--;
            }
        }
    }


    public function delete_item(Request $request)
    {
        $buyer = $request->user();
        $validator = Validator::make($request->all(),[
            'redemption_method'=>'required|numeric|between:1,2',
            'branch'=>'nullable|string',
            'item_code'=>'required',
        ]);
        if($validator->fails())
        {
            $response["message"]="Invalid request";
            $response["errors"]= $validator->errors();
            return response()->json($response);
        }
        $this->getUserCart($buyer->Enrollment_id)
        ->where('Item_code',$request->item_code)->where('Branch',$request->branch)
        ->Where('Redemption_method',$request->redemption_method)->delete();

        $cart_totals =$this->getUserCart($buyer->Enrollment_id)->sum('product.Billing_price_in_points');
        $response['status']= 1;
        $response['message']="Item Removed Successfully";
        $response['cart_total']=$cart_totals;
        return response()->json($response);
    }

}
