<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Mail\RedemptionMail;
use App\Models\Company;
use App\Traits\CheckoutTrait;
use App\Traits\ClientCartCheckoutTrait;
use App\Traits\RedeemPinTrait;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;

class ClientCheckoutController extends Controller
{
    use ClientCartCheckoutTrait,CheckoutTrait,RedeemPinTrait;
    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function checkout(Request $request)
    {
        $validator  = Validator::make($request->all(),[
            'cart'=>'required'
        ]);
        if($validator->fails())
        {
            return response()->json([
                'status'=>0,
                "message"=>"The cart items is/are required",
            ]);
        }
        $customer = $request->user();
        $company =  Company::with(['mail_configuration'])
            ->where('Company_id',$customer->Company_id)->first();
        $pin_validation = $company->Cust_redeem_pin_validation;
        if($pin_validation == 1)
        {
            $validator = Validator::make($request->all(),[
                'user_pin'=>'required',
            ]);
            if($validator->fails())
            {
                $response['status']=0;
                $response['message']="Invalid request";
                $response['errors']=$validator->errors();
                return response()->json($response);
            }

            if($this->hasTooManyAttempt($customer,3))
            {
                $this->fireAccountLockout($customer);
                $response['message']="You account has been blocked, because you tried wrong pin 3 times ";
                $response['status']=0;
                $response['attempt_left']=0;
                return response()->json($response);

            }
            else
            {
                if($request->user_pin != $customer->Pin)
                {
                    $remaining = $this->incrementRedeemPinAttempt($customer,3);
                    $response['message']="Incorrect pin, please try again, you have ".$remaining." attempt(s) left";
                    $response['status']=0;
                    $response['attempt_left']=$remaining;
                    return response()->json($response);
                }
            }
        }
       $client_items = collect($request->cart);
       $db_cart_items = $this->getCartItems($client_items);
       $invalid_items = $this->getInvalidClientItems($client_items,$db_cart_items);
        if(count($invalid_items)>0)
        {
            return response()->json([
                'status'=>0,
                "message"=>"The cart contains invalid items",
                'invalid_items'=>$invalid_items
            ]);
        }
        $valid_items = $this->constructCartItems($client_items,$db_cart_items);
        $delivery_items = $valid_items->where("Redemption_method",2);
        $total_price = $valid_items->sum("Amount");



        if($delivery_items->count() > 0)
        {
            $validator = Validator::make($request->all(),[
                'shipping_address_id'=>'required|numeric',
            ]);
            if($validator->fails())
            {
                return response()->json([
                    'status'=>0,
                    "errors"=>$validator->errors()
                ]);
            }
            $shipping_detail = DB::table("lsl_shipping_master")
                ->where([["Id",$request->shipping_address_id],["Enrollment_id",$customer->Enrollment_id]])->first();
            if($shipping_detail==null)
            {
                $response['message']="Shipping address not found";
                $response['status']=0;
                return response()->json($response);
            }
            $items_weight_id = $this->getItemsDeliveryPrice($delivery_items,$shipping_detail->State_id);
            $distinct_deliveries = $this->getDistinctStateDelivery($items_weight_id);
            $delivery_price_details = $this->getFinalDeliveryPrice($distinct_deliveries,$shipping_detail->State_id,
                $customer->Loyalty_programme_id);
            $delivery_price = $delivery_price_details["Delivery_price"];
            $grand_total = $delivery_price+$total_price;
            if($grand_total > $customer->walletBalance())
            {
                $response['message']="Insufficient funds to checkout your cart";
                $response['status']=0;
                $response['balance']=$customer->walletBalance();
                $response['cost']=$grand_total;
                return response()->json($response);
            }
            $customer->Current_balance -= $grand_total;
            $customer->Total_redeem_points += $grand_total;
            $customer->save();
            $order_no = $this->getOrderNumber($customer->Company_id);
            $this->saveOrderDetails($customer,$shipping_detail,$delivery_items,$order_no);
            $this->saveOrderDetailState($delivery_price_details["Delivery_items"],$order_no);
            $vouchers  = $this->saveTransaction($valid_items,$customer,$order_no,$shipping_detail->State_id,
                $shipping_detail->Full_address,$delivery_price);

            CustomMailJob::dispatch($company->mail_configuration->toArray(),$customer->User_email_id,
                new RedemptionMail($company,$customer,$valid_items,$order_no,$shipping_detail->Full_address,$vouchers));

        }else{
            if($total_price > $customer->walletBalance())
            {
                $response['message']="Insufficient funds to checkout your cart";
                $response['status']=0;
                $response['balance']=$customer->walletBalance();
                $response['cost']=$total_price;
                return response()->json($response);
            }
            $customer->Current_balance -= $total_price;
            $customer->Total_redeem_points += $total_price;
            $customer->save();
            $order_no = $this->getOrderNumber($customer->Company_id);
            $vouchers = $this->saveTransaction($valid_items,$customer,$order_no,null,
                null,0);

            CustomMailJob::dispatch($company->mail_configuration->toArray(),$customer->User_email_id,
                new RedemptionMail($company,$customer,$valid_items,$order_no,null,$vouchers));
        }
        $response['message']="Redemption has been completed Successfully, Please check your mail for details";
        $response['status']=1;
        return response()->json($response);

    }

    public function getDeliveryPrice(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'shipping_address_id'=>'required|numeric',
            'cart'=>'required'
        ]);
        if($validator->fails())
        {
            $response['errors']=$validator->errors();
            $response['status']=0;
            return response()->json($response);
        }
        $client_items = collect($request->cart);
        $db_cart_items = $this->getCartItems($client_items);
        $invalid_items = $this->getInvalidClientItems($client_items,$db_cart_items);
        if(count($invalid_items)>0)
        {
            return response()->json([
                'status'=>0,
                "message"=>"The cart contains invalid items",
                'invalid_items'=>$invalid_items
            ]);
        }
        $customer = $request->user();

        $valid_items = $this->constructCartItems($client_items,$db_cart_items);
        $delivery_items = $valid_items->where("Redemption_method",2);
        if($delivery_items->count() > 0)
        {
            $shipping_detail = DB::table("lsl_shipping_master")
                ->where("Id",$request->shipping_address_id)
                ->where("Enrollment_id",$customer->Enrollment_id)->first();
            if($shipping_detail==null)
            {
                $response['message']="Shipping address does not exist in our database";
                $response['status']=0;
                return response()->json($response);
            }
            $items_weight_id = $this->getItemsDeliveryPrice($delivery_items,$shipping_detail->State_id);
            $distinct_deliveries = $this->getDistinctStateDelivery($items_weight_id);
            $delivery_price_details = $this->getFinalDeliveryPrice($distinct_deliveries,$shipping_detail->State_id,
                $customer->Loyalty_programme_id);
        }else{

            $delivery_price_details['Delivery_price']=0;
        }
        $response['status']=1;
        $response["data"]=$delivery_price_details;
        return response()->json($response);
    }

    public function getCartTotal(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'cart'=>'required'
        ]);
        if($validator->fails())
        {
            $response['errors']=$validator->errors();
            $response['status']=0;
            return response()->json($response);
        }
        $client_items = collect($request->cart);
        $db_cart_items = $this->getCartItems($client_items);
        $invalid_items = $this->getInvalidClientItems($client_items,$db_cart_items);
        if(count($invalid_items)>0)
        {
            return response()->json([
                'status'=>0,
                "message"=>"The cart contains invalid items",
                'invalid_items'=>$invalid_items
            ]);
        }
        $customer = $request->user();
        $valid_items = $this->constructCartItems($client_items,$db_cart_items);
        $delivery_items = $valid_items->where("Redemption_method",2);
        $total_price = $valid_items->sum("Amount");
        if($delivery_items->count() > 0)
        {
            $shipping_addresses = DB::table("lsl_shipping_master")
                ->where("Enrollment_id",$customer->Enrollment_id)->get();
        }else{

            $shipping_addresses=null;
        }
        $response['status']=1;
        $response["Shipping_addresses"]=$shipping_addresses;
        $response["Items_total"] = $total_price;
        return response()->json($response);
    }
}
