<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Mail\EnrollmentMail;
use App\Traits\RedeemPinTrait;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use DB;
use App\Traits\CheckoutTrait;
use Validator;
use App\Models\Company;
use App\Mail\RedemptionMail;
use Illuminate\Support\Facades\Mail;

class DeliveryController extends Controller
{
    use CheckoutTrait,RedeemPinTrait;
    public function __construct()
    {
        $this->middleware("auth:api");
    }

    public function checkoutCart(Request $request)
    {
        $user = $request->user();
        //Get the company details of the request user

        $company = Company::with(['mail_configuration'])->find($user->Company_id);
        $pin_validation = $company->Cust_redeem_pin_validation;
        //check if the company allow pin validation during checkout
        if($pin_validation == 1)
        {
            $validator = Validator::make($request->all(),[
                'user_pin'=>'required',
            ]);
            if($validator->fails())
            {
                $response['status']=0;
                $response['message']="Invalid request";
                $response['errors']=$validator->errors();
                return response()->json($response);
            }

            if($this->hasTooManyAttempt($user,3))
            {
                $this->fireAccountLockout($user);
                $response['message']="You account has been blocked, because you tried wrong pin 3 times ";
                $response['status']=0;
                $response['attempt_left']=0;
                return response()->json($response);

            }
            else
            {
                if($request->user_pin != $user->Pin)
                {
                    $remaining = $this->incrementRedeemPinAttempt($user,3);
                    $response['message']="Incorrect pin, please try again, you have ".$remaining." attempt(s) left";
                    $response['status']=0;
                    $response['attempt_left']=$remaining;
                    return response()->json($response);
                }else{
                    $this->clearRedeemAttempt($user);
                }

            }
        }
        //Get invalid items due to some changes of Cart item, like branches, validity
        $invalid_items = $this->getInvalidItems($user->Enrollment_id);
        if(count($invalid_items) > 0)
        {
            $response['Sstatus']=0;
            $response['message']="There are some invalid items in the cart";
            $response['invalid_items']=$invalid_items;
            return response()->json($response);
        }
        $cart_items = $this->getItemsInCart($user->Enrollment_id);
        if(count($cart_items) <= 0)
        {
            $response['status']=0;
            $response['message']="There is no item in the cart";
            return response()->json($response);
        }
        $total_price = $cart_items->sum("Amount");
        //Get the Item that are meant for delivery
        $delivery_items = $this->getItemsForDelivery($user->Enrollment_id);
        if(count($delivery_items) > 0)
        {
            $validator = Validator::make($request->all(),[
                'shipping_address_id'=>'required|numeric',
                'delivery_fee'=>'required',
            ]);
            if($validator->fails())
            {
                $response['message']="Invalid request";
                $response['errors']=$validator->errors();
                return response()->json($response);
            }
            //Check if the delivery state exists
            $shipping_detail = DB::table("lsl_shipping_master")
                ->where("Id",$request->shipping_address_id)->first();
            if($shipping_detail==null)
            {
                $response['message']="State not found";
                $response['status']=0;
                return response()->json($response);
            }
            //get the delivery price in points
            $delivery_price_details = $this->getDeliveryPriceInPoints($user,$shipping_detail);
            $delivery_price = $delivery_price_details["Delivery_price"];
            $grand_total = $delivery_price+$total_price;
            if($grand_total > $user->walletBalance())
            {
                $response['message']="Insufficient funds to checkout your cart";
                $response['status']=0;
                return response()->json($response);
            }
            $user->Current_balance -= $grand_total;
            $user->Total_redeem_points += $grand_total;
            $user->save();
            $order_no = $this->getOrderNumber($user->Company_id);
            $this->saveOrderDetails($user,$shipping_detail,$delivery_items,$order_no);
            $this->saveOrderDetailState($delivery_price_details["Delivery_items"],$order_no);
            $vouchers  = $this->saveTransaction($cart_items,$user,$order_no,$shipping_detail->State_id,
            $shipping_detail->Full_address,$delivery_price);

            CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                new RedemptionMail($company,$user,$cart_items,$order_no,$shipping_detail->Full_address,$vouchers));
        }else{

            if($total_price > $user->walletBalance())
            {
                $response['message']="Insufficient funds";
                $response['status']=0;
                return response()->json($response);
            }
            $user->Current_balance -= $total_price;
            $user->Total_redeem_points += $total_price;
            $user->save();
            $order_no = $this->getOrderNumber($user->Company_id);
            $vouchers = $this->saveTransaction($cart_items,$user,$order_no,null,
            null,0);

            CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                new RedemptionMail($company,$user,$cart_items,$order_no,null,$vouchers));
        }
        $response['message']="Redemption has been completed Successfully, Please check your mail for details";
        $response['status']=1;
        return response()->json($response);
    }

    public function getDeliveryPrice(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'state_id'=>'required|numeric'
        ]);
        if($validator->fails())
        {
            $response['errors']=$validator->errors();
            $response['status']=0;
            return response()->json($response);
        }
        $shipping_detail = DB::table("lsl_shipping_master")
            ->where("Id",$request->shipping_address_id)->first();
        if($shipping_detail==null)
        {
            $response['message']="Shipping Address does not exist in our database";
            $response['status']=0;
            return response()->json($response);
        }
        $user = $request->user();
        $final_prices = $this->getDeliveryPriceInPoints($user,$shipping_detail);
        $response['message']="Request Successful";
        $response['status']=1;
        $response["data"]=$final_prices;
        return response()->json($response);
    }
}
