<?php

namespace App\Http\Controllers\API;

use App\Jobs\CustomMailJob;
use App\Mail\Promo\PromoMail;
use App\Mail\RedemptionMail;
use App\Models\Company;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use DB;
use Illuminate\Support\Facades\Mail;
use Validator;

class PromoCodeController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function getPoints(Request $request)
    {
        $validators = Validator::make($request->all(),[
            'promo_code'=>'required'
        ]);
        if($validators->fails())
        {
            $response['status'] = 0;
            $response['message'] = "Promo code is required";
            $response['errors'] = $validators->errors();
            return response()->json($response);
        }
        $user = $request->user();
        $codes = DB::table('lsl_promo_campaign as pro')
        ->leftJoin('lsl_campaign_master as cam','pro.Campaign_id','cam.Campaign_id')
        ->leftJoin('lsl_loyalty_program_master as prog','pro.Company_id','=','prog.Company_id')
        ->where('pro.Company_id',$user->Company_id)
        ->where('prog.Loyalty_program_id',$user->Loyalty_programme_id)
        ->where('pro.Promo_code_status',0)->where('pro.Promo_code',$request->promo_code)
        ->select('cam.Campaign_type','prog.Loyalty_program_name','cam.Campaign_sub_type','pro.Campaign_id',
        'pro.Points','pro.Date','pro.Promo_code')->get();
        if(count($codes) == 0)
        {
            $response['status'] = 0;
            $response['message'] = "Invalid Promo code, Check and try again";
            return response()->json($response);
        }
        $now = date("Y-m-d");
        $code = $codes->where('Date','<=',$now)->first();
        if($code==null)
        {
            $response['status'] = 0;
            $response['message'] = "Promo code is no longer valid";
            return response()->json($response);
        }
        $user->Current_balance +=$code->Points;
        $user->save();

        $transaction_id = $this->savePromoTransaction($user,$code->Points);
        DB::table('lsl_transaction_child')->insert([
            'Enrollment_id'=>$user->Enrollment_id,
            'Transaction_id'=>$transaction_id,
            'Campaign_id'=>$code->Campaign_id,
            'Campaign_type'=>$code->Campaign_type,
            'Campaign_sub_type'=>$code->Campaign_sub_type,
            'Reward_points'=>$code->Points,
            'Company_id'=>$user->Company_id,
            'Date'=>date("Y-m-d",time()),
        ]);
        DB::table('lsl_promo_campaign')->where('Promo_code',$code->Promo_code)->update(['Promo_code_status'=>1]);

        $company =  Company::with(['mail_configuration'])->where('Company_id',$user->Company_id)->first();;

        $response['status'] = 1;
        $response['message'] = "Congratulations you have been credited with ".$code->Points." Points";
        $response['current_balance'] = $user->Current_balance;

        CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
            new PromoMail($user,$code->Points,13,$code->Promo_code,$code->Loyalty_program_name,$company));

        return response()->json($response);
    }

    private function savePromoTransaction($user,$points)
    {
        return DB::table("lsl_transaction")->insertGetId([
            'Transaction_type_id'=>13,'Company_id'=>$user->Company_id,'Child_company_id'=>0,'Member1_enroll_id'=>$user->Enrollment_id,
            'Membership1_id'=>$user->Membership_id,'Branch_user_id'=>0,'Transaction_date'=>date("Y-m-d H:m:s",time()),
            'Branch_user_pin'=>"null",'Item_code'=>"","Merchandize_partner_id"=>0,'Merchandize_partner_branch'=>"",
            'Quantity'=>0,"Company_transaction_Type"=>"","Transaction_channel_code"=>"","Product_group_code"=>"","Product_brand_code"=>"",'Redeem_amount'=>0,
            'Redeem_points'=>0,"Transaction_amount"=>0.00,"Product_cost"=>0.00,"Redeem_amount_value"=>0.00,
            "Total_shipping_points"=>0.00,"Total_shipping_cost"=>0.00,"Bonus_points"=>$points,"Loyalty_points"=>0.00,
            "Knockout_points"=>0.00,"Balance_to_pay"=>0.00,'Payment_type_id'=>0,'Partner_billing_price'=>0,"Transfer_points"=>0,
            "Account_no"=>0,'Partner_cost_payable'=>0,'Loyalty_program_id'=>0,"Member2_enroll_id"=>0,"Membership2_id"=>0,
            'Shipping_points'=>0,'Source'=>"API",'Total_shipping_value'=>0,'Branch_code'=>"",'Remarks'=>"",
            'Delivery_flag'=>1,'Delivery_method'=>0,'Voucher_status'=>0,'Order_status'=>0,'Voucher_no'=>"",'Order_no'=>0,
            'Report_status'=>0,'Field_5_name'=>"",'Delivery_state_id'=>0
       ],"Transaction_id");
    }
}
