<?php

namespace App\Http\Controllers\API;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\ShippingAddress;

class ShippingController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'first_name'=>'required',
            'last_name'=>'required',
            'state_id'=>'required',
            'city_id'=>'required',
            'address'=>'required',
            'contact_phone'=>'required',
            'email'=>'required'
        ]);
        if($validator->fails())
        {
            $response['status']=0;
            $response['message']='Invalid request';
            $response['errors'] = $validator->errors();
            return response()->json($response);
        }
        $city = \DB::table("lsl_city_master as city")
        ->leftJoin("lsl_state_master as state","state.State_id","=","city.State_id")
        ->Select("city.City_name","state.State_name")
        ->where("City_id",$request->city_id)->where("city.State_id",$request->state_id)->first();
        if($city == null)
        {
            $response['message']='Invalid state_id or city_id';
            $response['status'] =0;
            return response()->json($response);
        }
        $user = $request->user();
        $address = ShippingAddress::create([
            'First_name'=>$request->first_name,
            'Last_name'=>$request->last_name,
            'State_id'=>$request->state_id,
            'City_id'=>$request->city_id,
            'Address'=>$request->address,
            'Full_address'=>$request->address." ".$city->City_name." ".$city->State_name,
            'Contact_phone'=>$request->contact_phone,
            'Contact_email'=>$request->email,
            'Enrollment_id'=>$user->Enrollment_id
        ]);
        $response['status']=1;
        $response['message']="A new shipping details has been created";
        $response['shipping_details']= $address;
        return response()->json($response);
    }

    public function update(Request $request,$id)
    {
        $validator = Validator::make($request->all(),[
            'id'=>'required',
            'first_name'=>'required',
            'last_name'=>'required',
            'state_id'=>'required',
            'city_id'=>'required',
            'address'=>'required',
            'contact_phone'=>'required',
            'email'=>'required'
        ]);
        if($validator->fails())
        {
            $response['status']=0;
            $response['message']='Invalid request';
            $response['errors'] = $validator->errors();
            return response()->json($response);
        }

        $city = \DB::table("lsl_city_master as city")
        ->leftJoin("lsl_state_master as state","state.State_id","=","city.State_id")
        ->Select("city.City_name","state.State_name")
        ->where("City_id",$request->city_id)->where("city.State_id",$request->state_id)->first();
        if($city == null)
        {
            $response['message']='Invalid state_id or city_id';
            $response['status'] =0;
            return response()->json($response);
        }
        $user = $request->user();

        $address = ShippingAddress::where("Id",$request->id)->where("Enrollment_id",$user->Enrollment_id)->first();
        if($address == null)
        {
            $response['status']=0;
            $response['message']="Shpping address not found";
            return response()->json($response);
        }


        $address->fill([
            'First_name'=>$request->first_name,
            'Last_name'=>$request->last_name,
            'State_id'=>$request->state_id,
            'City_id'=>$request->city_id,
            'Address'=>$request->address,
            'Full_address'=>$request->address." ".$city->City_name." ".$city->State_name,
            'Contact_phone'=>$request->contact_phone,
            'Contact_email'=>$request->email,
            'Enrollment_id'=>$user->Enrollment_id
        ]);
        $address->save();
        $response['status']=1;
        $response['message']="Shipping detail has been updated successfully";
        $response['shipping_details']= $address;
        return response()->json($response);
    }
}
