<?php

namespace App\Http\Controllers\API;

use App\Models\Company;
use App\Traits\TransferTrait;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class TransferController extends Controller
{
    use TransferTrait;
    public function __construct()
    {
        $this->middleware('auth:api');
    }

    public function transferPoints(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'membership_id'=>'required',
            'amount'=>'required|numeric'
        ]);
        if($validator->fails())
        {
            $response['status'] = 0;
            $response['message']="Invalid request";
            $response['errors']=$validator->errors();
            return response()->json($response);
        }
        $user = $request->user();
        $amount = $request->amount;
        if(($user->walletBalance()) < $amount)
        {
            $response['status'] = 0;
            $response['message']="Insufficient balance to transfer the points";
            return response()->json($response);
        }

        $recipient = $this->getRecipient($request->membership_id,$user->Loyalty_programme_id);
        if($recipient == null)
        {
            $response['status'] = 0;
            $response['message']="No member is found";
            return response()->json($response);
        }
        if($recipient->Active_flag==0)
        {
            $response['status'] = 0;
            $response['message']="Member is currently not active";
            return response()->json($response);
        }
        if($recipient->Membership_id==$user->Membership_id)
        {
            $response['status'] = 0;
            $response['message']="You cannot transfer credit to your self";
            return response()->json($response);
        }
        if($recipient->Pin_no_applicable == 1)
        {
            $validator = Validator::make($request->all(),[
                'pin'=>'required',
            ]);
            if($validator->fails())
            {
                $response['status'] = 0;
                $response['message']="User Pin is required request";
                return response()->json($response);
            }
        }
        $company = Company::with(['mail_configuration'])->where('Company_id',$user->Company_id)->first();
        $this->debitAccount($user->Enrollment_id,$recipient,$amount,$company);
        $this->creditAccount($user,$recipient->Enrollment_id,$amount,$company);

        $response['status'] = 1;
        $response['message']="Your transfer of ".$amount." is successful";
        $response['amount']= $amount;
        return response()->json($response);
    }

}
