<?php

namespace App\Traits;

use App\Models\TempCart;
use App\Models\Company;
use DB;
use Validator;
use Carbon\Carbon;

trait CheckoutTrait
{
    use KnockoutTrait;
    protected function saveOrderDetailState($delivery_items,$order_id)
    {
        foreach ($delivery_items as $delivery_item) {

            $temp_deliveries = [
                'order_id'=>$order_id,
                'from_state'=>$delivery_item['State_from'],
                'to_state'=>$delivery_item["State_to"],
                'total_state_weight'=>$delivery_item["Weight"],
                'state_price'=>$delivery_item["Delivery_price"],
                'create_date'=>date("Y-m-d H:m:s",time())
            ];
         }
         $result = \DB::table('delivery_order_details_state')->insert($temp_deliveries);
    }

    protected function getTrackID(){
        if (function_exists('com_create_guid')){
            return com_create_guid();
        }else{
            mt_srand((double)microtime()*10000);//optional for php 4.2.0 and up.
            $charid = (md5(uniqid(rand(), true)));
            $hyphen = chr(45);// "-"
            $uuid = substr($charid, 0, 8).$hyphen
                .substr($charid, 8, 4).$hyphen
                .substr($charid,12, 4).$hyphen
                .substr($charid,16, 4).$hyphen
                .substr($charid,20,12);
            return $uuid;
        }
    }

    protected function getDeliveryPriceInPoints($user,$state)
    {
        $cart_items = $this->getItemsForDelivery($user->Enrollment_id);
        if(count($cart_items) > 0)
        {
            $items_weight_id = $this->getItemsDeliveryPrice($cart_items,$state->State_id);
            $distinct_deliveries = $this->getDistinctStateDelivery($items_weight_id);
            $final_prices = $this->getFinalDeliveryPrice($distinct_deliveries,$state->State_id,
                            $user->Loyalty_programme_id);
        }else{
            $final_prices = [
                'Delivery_price'=>0
            ];
        }

        return  $final_prices;
    }

    protected function getItemsForDelivery($user_id)
    {
      return  TempCart::with("branches")->join('lsl_company_merchandise_catalogue as 
                cat', 'lsl_temp_cart.Item_code','=','cat.Company_Merchandize_item_code')
                ->select(DB::raw('count(*) as Quantity,sum(cat.Billing_price_in_points) as Amount, cat.Billing_price_in_points as Price,sum(cat.Weight) as Weight, cat.Weight as Unit_weight,cat.Weight_unit_id
                 as Weight_unit_id'),"Item_code","cat.Merchandize_item_name as Item_name","cat.Partner_id",
                 "cat.Merchandize_category_id as Category_id")->where("Enrollment_id",$user_id)->where("Redemption_method",2)
                ->groupBy(["Redemption_method","Branch","Item_code","Remark","cat.Weight_unit_id","cat.Weight",
                "cat.Merchandize_item_name","cat.Billing_price_in_points","cat.Partner_id","cat.Merchandize_category_id"])->get();
    }

    protected function getItemsDeliveryPrice($items,$state_id)
    {
        foreach($items as $item)
        {
            $states = $item->branches->pluck("State_id");
            $delivery_pricing=   DB::table("lsl_weight_master")
                ->join("lsl_delivery_price_master as div","div.Weight_id","lsl_weight_master.Weight_id")
                ->where("Weight_start_range","<=",$item->Weight)->where("Weight_end_range",">=",$item->Weight)
                ->WhereIn("State_from", $states)->Where("State_to",$state_id)->where("Weight_unit_id",$item->Weight_unit_id)
                ->Select("div.Weight_range_price as Delivery_price","lsl_weight_master.Weight_id","State_from")
                ->orderBy("div.Weight_range_price")->first();
            $response[]=[
                "Item_code"=>$item->Item_code,
                "Weight_id"=>$delivery_pricing->Weight_id,
                "Quantity"=>$item->Quantity,
                "Weight"=>$item->Weight,
                "State_from"=>$delivery_pricing->State_from,
                "Delivery_price"=>$delivery_pricing->Delivery_price
            ];
        }

        return $response;
    }

    protected function getDistinctStateDelivery($items_array)
    {
        $response=[];
        foreach($items_array as $item)
        {
            if(array_key_exists($item["State_from"],$response))
            {
                $response[$item["State_from"]]["Weight"] += $item["Weight"];
            }else{
                $response[$item["State_from"]]= [
                    "Weight"=>$item["Weight"],
                    "State_from"=>$item["State_from"],
                ];
            }
        }

        return $response;
    }

    protected function getFinalDeliveryPrice($items_array,$state_to,$loyalty_program_id)
    {
        $total_delivery_price = 0;
        foreach($items_array as $item)
        {
            $db_item= DB::table("lsl_weight_master")
            ->join("lsl_delivery_price_master as div","div.Weight_id","lsl_weight_master.Weight_id")
            ->where("Weight_start_range","<=",$item["Weight"])->where("Weight_end_range",">=",$item["Weight"])
            ->Where("State_from", $item["State_from"])->Where("State_to",$state_to)
            ->Select("div.Weight_range_price as Delivery_price","lsl_weight_master.Weight_id","State_to","State_from")
            ->orderBy("div.Weight_range_price")->first();

            $delivery_price_in_point = $this->convertToPoints($db_item->Delivery_price,$loyalty_program_id);
            $delivery_items[]=[
                "Delivery_price"=>$delivery_price_in_point,
                "State_to"=>$db_item->State_to,
                "State_from"=>$db_item->State_from,
                "Weight"=>$item["Weight"]
            ];
            $total_delivery_price +=$delivery_price_in_point;
        }
        $response['Delivery_price']=$total_delivery_price;
        $response['Delivery_items']=$delivery_items;

        return $response;
    }

    protected function convertToPoints($price,$programme_id)
    {
        $program = DB::table('lsl_loyalty_program_master')
                    ->where('Loyalty_program_id',$programme_id)->first();

        if($program != null)
        {
          return ceil($price * $program->Points_value_definition);
        }
        return null;
    }

    protected function getItemsInCart($user_id)
    {
        return TempCart::join('lsl_company_merchandise_catalogue as cat', 'lsl_temp_cart.Item_code','=',
        'cat.Company_Merchandize_item_code')
        ->join('lsl_partner_master as pat','pat.Partner_id','=','cat.Partner_id')
       ->select(DB::raw('count(*) as Quantity,sum(cat.Billing_price_in_points) as Amount'),"Item_code",
       "Redemption_method","Branch","Remark","pat.Partner_name","pat.Api_redemption","pat.Partner_id",'pat.Partner_address',
       "cat.Billing_price_in_points as Price","cat.Cost_payable_to_partner","cat.Cost_in_points","cat.Merchandize_item_name as Item_name")
       ->where("Enrollment_id",$user_id)->groupBy(["Redemption_method","pat.Partner_name","pat.Api_redemption",
       "cat.Merchandize_item_name","pat.Partner_address","Branch","Item_code","Remark","pat.Partner_id","cat.Billing_price_in_points",
       "cat.Cost_payable_to_partner","cat.Cost_in_points"])->get();
    }

    protected function getInvalidItems($user_id)
    {
       $cart_items = TempCart::with('product')->with("branch_codes")
       ->join('lsl_company_merchandise_catalogue as cat', 'lsl_temp_cart.Item_code','=',
       'cat.Company_Merchandize_item_code')
      ->select(DB::raw('count(*) as Quantity,sum(cat.Billing_price_in_points) as Amount'),"Item_code","Redemption_method","Branch","Remark")
      ->where("Enrollment_id",$user_id)->groupBy(["Redemption_method","Branch","Item_code","Remark"])
      ->get();
      $now = Carbon::now()->toDateString();
        $invalid_items=[];
       foreach($cart_items as $item)
       {
           if($item->Redemption_method != $item->product->Delivery_method && $item->product->Delivery_method != 3)
           {
                $invalid_items[]=[
                    'Item_code'=>$item->Item_code,
                    "Quantity"=>$item->Quantity,
                    "Branch"=>$item->Branch,
                    "Redemption_method"=>$item->Redemption_method,
                    "Remark"=>$item->Remark,
                    "Reason"=>"Redemption method not Supported for this item"
                ];
           }elseif($item->Redemption_method==1 &&
           !in_array($item->Branch,$item->branch_codes->pluck("Branch_code")->toArray()))
           {
                $invalid_items[]=[
                    'Item_code'=>$item->Item_code,
                    "Quantity"=>$item->Quantity,
                    "Branch"=>$item->Branch,
                    "Remark"=>$item->Redemption_method,
                    "Remark"=>$item->Remark,
                    "Reason"=>"Invalid Branch"
                ];
           }elseif($item->Remark==null && $item->product->Enable_remark==1)
           {
                $invalid_items[]=[
                    'Item_code'=>$item->Item_code,
                    "Quantity"=>$item->Quantity,
                    "Branch"=>$item->Branch,
                    "Remark"=>$item->Remark,
                    "Redemption_method"=>$item->Redemption_method,
                    "Reason"=>"Invalid Remark"
                ];
           }elseif($item->product->Active_flag==0 ||
           ($item->product->Valid_from >= $now && $item->product->Valid_till <= $now))
           {
                $invalid_items[]=[
                    'Item_code'=>$item->Item_code,
                    "Quantity"=>$item->Quantity,
                    "Branch"=>$item->Branch,
                    "Redemption_method"=>$item->Redemption_method,
                    "Remark"=>$item->Remark,
                    "Reason"=>"This item is no longer valid"
                ];
           }
       }
        return $invalid_items;
    }


    protected function saveTransaction($items,$user,$order_no,$state_id,$full_address,$delivery_fee)
    {
        $user_points = $this->getUserPoints($user->Company_id,$user->Enrollment_id);
        foreach($items as $item)
        {
            $voucher_no = $this->getRandomString();
           $transaction_id =  $this->saveTransactionObject($user,$item,$full_address,$state_id,
           $voucher_no,$order_no,$delivery_fee);
           $vouchers[$item->Item_code] =  $voucher_no;
           $this->deductUserPoints($user_points,$item->Amount,$user->Enrollment_id,
                                          $transaction_id,$user->Company_id,0,$voucher_no);
            \DB::table('lsl_temp_cart')->where('Item_code',$item->Item_code)
            ->where('Redemption_method',$item->Redemption_method)->where('Branch',$item->Branch)
            ->where("Remark",$item->Remark)->where('Enrollment_id',$user->Enrollment_id)->delete();
        }

        $api_redemption_items = $items->where("Api_redemption",1);

        if(count($api_redemption_items) > 0)
        {
            $this->saveApiRedemption($api_redemption_items,$vouchers,$transaction_id,$user);
        }

        return $vouchers;




    }



    protected function saveTransactionObject($user,$item,$full_address,$state_id,$voucher_no,$order_no,$delivery_fee)
    {
       $user_contact_details = "Full Name: ".$user->First_name." ".$user->Last_name;
       $user_contact_details .= "Phone_no: ".$user->Phone_no;
       $user_contact_details .= "Email_address: ".$user->User_email_id;

       return \DB::table("lsl_transaction")->insertGetId([
            'Transaction_type_id'=>2,'Company_id'=>$user->Company_id,'Child_company_id'=>0,'Member1_enroll_id'=>$user->Enrollment_id,
            'Membership1_id'=>$user->Membership_id,'Branch_user_id'=>0,'Transaction_date'=>date("Y-m-d H:m:s",time()),'Branch_user_pin'=>null,
            'Item_code'=>$item->Item_code,'Merchandize_partner_id'=>$item->Partner_id,'Merchandize_partner_branch'=>$item->Branch,
            'Quantity'=>$item->Quantity,"Company_transaction_Type"=>"","Transaction_channel_code"=>"","Product_group_code"=>"","Product_brand_code"=>"",'Redeem_amount'=>$item->Amount,
            'Redeem_points'=>$item->Price,"Transaction_amount"=>0.00,"Product_cost"=>0.00,"Redeem_amount_value"=>0.00,
            "Total_shipping_points"=>0.00,"Total_shipping_cost"=>0.00,"Bonus_points"=>0.00,"Loyalty_points"=>0.00,
            "Knockout_points"=>0.00,"Balance_to_pay"=>0.00,'Payment_type_id'=>0,'Partner_billing_price'=>$item->Cost_in_points,"Transfer_points"=>0,
            "Account_no"=>0,'Partner_cost_payable'=>$item->Cost_payable_to_partner,'Loyalty_program_id'=>0,"Member2_enroll_id"=>0,"Membership2_id"=>0,
            'Shipping_points'=>$item->Redemption_method==1?0:$delivery_fee,'Source'=>"API",'Total_shipping_value'=>0,'Branch_code'=>$item->Branch,'Remarks'=>$item->Redemption_method==1?"":$full_address,
            'Delivery_flag'=>1,'Delivery_method'=>$item->Redemption_method,'Voucher_status'=>141,'Order_status'=>146,'Voucher_no'=>$voucher_no,'Order_no'=>$order_no,
            'Report_status'=>0,'Field_5_name'=>$user_contact_details,'Delivery_state_id'=>$item->Redemption_method==1?$state_id:0
       ],"Transaction_id");
    }


    protected function getRandomString($length = 16)
    {
        $characters = 'A123B56C89DE4FGH7IJKL8MNOP';
        $string = '';
        for ($i = 0; $i < $length; $i++)
        {
            $string .= $characters[mt_rand(0, strlen($characters) - 1)];
        }
        return $string;
    }

    protected function saveOrderDetails($user,$shipping_details,$delivery_items,$order_no)
    {
        $order_id = \DB::table('delivery_order')->insertGetId([
            'full_address'=>$shipping_details->Full_address,'first_name'=>$shipping_details->First_name,'Company_id'=>$user->Company_id,
            'last_name'=>$shipping_details->Last_name,'email'=>$shipping_details->Contact_email,'phone_no'=>$shipping_details->Contact_phone,
            'line_address'=>$shipping_details->Address,'state_id'=>$shipping_details->State_id,'city_id'=>$shipping_details->City_id,
            'order_no'=>$order_no,'track_no'=>$this->getTrackID(),'invoice_no'=>"D".$user->Company_id.$order_no,'price'=>$delivery_items->sum("Amount")
        ],'order_id');

        foreach ($delivery_items as $delivery_item) {
           $temp_deliveries = [
               'order_id'=>$order_id,
               'state_detail_id'=>$shipping_details->State_id,
               'item_name'=>$delivery_item->Item_name,
               'item_weight'=>$delivery_item->Unit_weight,
               'item_quantity'=>$delivery_item->Quantity,
               'create_date'=>date("Y-m-d H:m:s",time())
           ];
        }
        $result = \DB::table('delivery_order_details_item')->insert($temp_deliveries);
    }

    protected function saveApiRedemption($cart_items,$vouchers,$transaction_id,$user)
    {
        foreach($cart_items as $item)
        {
            $temps[] =[
                "Transaction_id"=>$transaction_id,"Company_id"=>$user->Company_id,"Membership_id"=>$user->Membership_id,
                "Full_name"=>$user->Last_name." ".$user->First_name,"Phone_no"=>$user->Phone_no,"Email_id"=>$user->User_email_id,
                "Remark"=>$item->Remark,"Transaction_date"=>date("Y-m-d H:m:s",time()),"Item_name"=>$item->Item_name,
                "Quantity"=>$item->Quantity,"Redeem_points"=>$item->Amount,"Redeem_points_value"=>$item->Amount,
                "Total_redeemed_points"=>$item->Amount,"Redeem_amount_value"=>$item->Amount,"Merchandize_partner_id"=>$item->Partner_id
                ,"Merchandize_category_id"=>$item->Category_id,"Voucher_no"=>$vouchers[$item->Item_code],"Item_code"=>$item->Item_code,
                "Merchandize_partner_branch"=>$item->Redemption_method=1?null:$item->Branch
            ];
        }
        \DB::table('lsl_redeemption_contactdetails')->insert($temps);
    }


    protected function getOrderNumber($company_id)
    {
        $company = Company::where('Company_id',$company_id)->first();
        $order_no = $company->Order_no_series;
        $company->Order_no_series++;
        $company->save();
        return $order_no;
        $i=0;
        while($i<1)
        {
            $order_no = $company->Order_no_series;
            $company->Order_no_series++;
            $company->save();
            $count =\DB::table('delivery_order')->where("Company_id",$company->Company_id)
            ->where('order_no',$order_no)->count();
            if($count < 1)
            {
                return $order_no;
            }

        }
    }
}
