<?php

namespace App\Traits;

use App\Models\MerchandizeCatalogue;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;

trait ClientCartCheckoutTrait
{
    public function getCartItems($client_cart)
    {
        $items_code = $client_cart->pluck('Item_code');
       return MerchandizeCatalogue::with('branches')->with('company_partner')
                    ->whereIn('Merchandize_item_code',$items_code)->get();
    }

    public function getInvalidClientItems($client_cart,$db_cart_items)
    {
        $invalid_items=[];
        foreach($client_cart as $item)
        {
            $branch_code = $item["Branch_code"];
            $db_item = $db_cart_items->where('Merchandize_item_code',$item["Item_code"])->first();
            if($db_item == null)
            {
                $invalid_items[] = [
                    "Item_code"=>$item["Item_code"],
                    "Message"=>"Item does not exist"
                ];
            }else if(!$db_item->branches->where("Branch_code",$branch_code)->count())
            {
                $invalid_items[] = [
                    "Item_code"=>$item["Item_code"],
                    "Message"=>"Invalid Item Branch"
                ];
            }else if($item["Quantity"] <= 0)
            {
                $invalid_items[] = [
                    "Item_code"=>$item["Item_code"],
                    "Message"=>"Item can not be less than 1"
                ];
            }else if(($item["Redemption_method"] != $db_item->Delivery_method)
                && $db_item->Delivery_method!=3)
            {
                $invalid_items[] = [
                    "Item_code"=>$item["Item_code"],
                    "Message"=>"Invalid Redemption method"
                ];
            }
        }
        return $invalid_items;
    }

    protected function constructCartItems($client_cart,$db_cart_items)
    {
        $constructed_items=new Collection();
        foreach($client_cart as $item)
        {
            $branch_code = $item["Branch_code"];
            $db_item = $db_cart_items->where('Merchandize_item_code',$item["Item_code"])->first();
            if($db_item != null)
            {
                if ($db_item->branches->where("Branch_code", $branch_code)->count())
                {
                    $constructed_items->push(
                      (object)([
                          "Item_code" => $item["Item_code"],
                          "Branch_code"=>$item["Branch_code"],
                          "Quantity"=>$item["Quantity"],
                          "Price"=>$db_item->Billing_price_in_points,
                          "Amount"=>$db_item->Billing_price_in_points*$item["Quantity"],
                          "Redemption_method"=>$item["Redemption_method"],
                          "Remark"=>$item["Remark"],
                          "Weight"=>$db_item->Weight*$item["Quantity"],
                          'Unit_weight'=>$db_item->Weight,
                          'Partner_id'=>$db_item->Partner_id,
                          "Branch"=>$item["Branch_code"],
                          "Cost_in_points"=>$db_item->Cost_in_points,
                          "Weight_unit_id"=>$db_item->Weight_unit_id,
                          "branches"=>$db_item->branches,
                          "Cost_payable_to_partner"=>$db_item->Cost_payable_to_partner,
                          "Item_name"=>$db_item->Merchandize_item_name,
                          "Partner_name"=>$db_item->company_partner->Partner_name,
                          "Partner_address"=>$db_item->company_partner->Partner_address,
                          "Api_redemption"=>$db_item->company_partner->Api_redemption,

                      ])
                    );
                }
            }
        }
        return $constructed_items;
    }


}
