<?php

namespace App\Traits;

use Illuminate\Support\Facades\DB;

trait StatementTrait
{
    protected function getUserStatement($membership_id,$company_id)
    {
        return DB::table('lsl_transaction as t')->whereIn('Transaction_type_id',[1,2,3,4,6,7,14,30])
            ->leftJoin('lsl_pos_inventory_master as inventory',function($query){
                $query->on('t.Item_code','=','inventory.Item_code')
                    ->where('t.transaction_type_id','=',1)
                    ->where('t.Loyalty_points','>',0);
            })
            ->leftJoin('lsl_company_merchandise_catalogue as cat',function($query){
                $query->on('t.Item_code','=','cat.Merchandize_item_code')
                    ->where('t.Transaction_type_id','2');
            })->leftJoin('lsl_company_merchandise_catalogue as cat2',function($query){
                $query->on('t.Item_code','=','cat2.Merchandize_item_code')
                    ->where('t.Transaction_type_id','30');
            })->select("t.Membership1_id","t.Quantity","t.Bonus_points","t.Transaction_date",
           "t.Transaction_type_id","t.Transfer_points","t.Loyalty_points","t.Voucher_no","t.Branch_code","t.Order_no",
           "t.Transaction_amount","t.Total_shipping_points","t.Transaction_date","t.Delivery_method","t.Remarks","t.Benefit_description",
           DB::raw('(Quantity *Redeem_points) as Total_redeem_points,
            (CASE WHEN t.Transaction_type_id=1 THEN `inventory`.`Item_name`
           WHEN t.Transaction_type_id=2 OR t.Transaction_type_id=30 THEN `cat`.`Merchandize_item_name`  END) as Item_name'))
           ->where("t.Membership1_id",$membership_id)->where("t.Company_id",$company_id)
            ->where('Transaction_status',0)->orderBy("t.Transaction_id","desc");
    }

    protected function getUserStatementSummary($membership_id,$company_id)
    {
        return DB::table('lsl_transaction as t')->whereIn('Transaction_type_id',[1,2,3,4,6,7,14,30])
            ->leftJoin('lsl_pos_inventory_master as inventory',function($query){
                $query->on('t.Item_code','=','inventory.Item_code')
                    ->where('t.transaction_type_id','=',1)
                    ->where(function($query2){
                        $query2->where('t.Loyalty_points','>',0)->orWhere('t.Redeem_points','>',0);
                    });
            })
            ->leftJoin('lsl_company_merchandise_catalogue as cat',function($query){
                $query->on('t.Item_code','=','cat.Merchandize_item_code')
                    ->where('t.Transaction_type_id','2');
            })->leftJoin('lsl_company_merchandise_catalogue as cat2',function($query){
                $query->on('t.Item_code','=','cat2.Merchandize_item_code')
                    ->where('t.Transaction_type_id','30');
            })->select("t.Transaction_date","t.Transaction_type_id",
                DB::raw('(CASE WHEN t.Transaction_type_id=1 AND (t.Loyalty_points>0 OR t.Redeem_points > 0) THEN 
                  CONCAT_WS("","Points gained from: ",`inventory`.`Item_name`)
                   WHEN `t`.`Transaction_type_id`=2 AND `t`.`Delivery_method`=1 THEN CONCAT_WS("","Pick up ",`cat`.`Merchandize_item_name`,
                  " (Qty: ",`t`.`Quantity`,") from ",`t`.`Branch_code`)
                   WHEN t.Transaction_type_id=2 AND t.Delivery_method=2 THEN CONCAT_WS("","Deliver ",`cat`.`Merchandize_item_name`,
                  " (Qty: ",`t`.`Quantity`,") to ",`t`.`Branch_code`) 
                  WHEN t.Transaction_type_id=3 AND t.Membership2_id<>"" THEN CONCAT_WS("","Transfer from ",`t`.`Membership2_id`)
                  WHEN t.Transaction_type_id=3 OR Transaction_type_id=4 AND t.Membership2_id="" THEN `t`.`Benefit_description`
                  WHEN t.Transaction_type_id=6 THEN CONCAT_WS("","Transfer to ",`t`.`Membership2_id`)
                  WHEN t.Transaction_type_id=8 THEN CONCAT_WS("",`t`.`Transaction_amount`," points has Expired")
                  WHEN t.Transaction_type_id=7 THEN CONCAT_WS("",`t`.`Transaction_amount`," points has been forfeited")
                  WHEN t.Transaction_type_id=30 THEN CONCAT_WS("","Refund on ",`cat`.`Merchandize_item_name`)
                  WHEN t.Transaction_type_id=14 THEN "Auction Bidding"
                  ELSE "-" END) AS Description,
                   
                  (CASE WHEN t.Transaction_type_id=1 AND t.Loyalty_points>0 THEN `t`.`Loyalty_points`
                  WHEN t.Transaction_type_id IN (4,30,3) THEN `t`.`Bonus_points`
                  ELSE "-" END) AS Points_gained,
               
                 (CASE 
                 WHEN t.Transaction_type_id IN (7,8,14) THEN `t`.`Transaction_amount`
                 WHEN t.Transaction_type_id = 2 THEN (Quantity *Redeem_points)
                 WHEN t.Transaction_type_id=1 AND t.Redeem_points > 0 THEN `t`.`Redeem_points`
                  WHEN t.Transaction_type_id=6 THEN `t`.`Transfer_points`
                 ELSE "-" END) AS Points_used,
               
                  (CASE WHEN t.Transaction_type_id=1 AND (t.Loyalty_points>0 OR t.Redeem_points>0) THEN "Transaction"
                  WHEN t.Transaction_type_id=2 THEN "Redemption" 
                  WHEN t.Transaction_type_id=3 AND t.Membership2_id<>"" THEN "Transfer In"
                  WHEN t.Transaction_type_id=3 AND t.Membership2_id="" THEN "Points Credit"
                  WHEN t.Transaction_type_id=4 THEN "Points Debit"
                  WHEN t.Transaction_type_id=6 THEN "Transfer out"
                  WHEN t.Transaction_type_id=8 THEN "points Expired"
                  WHEN t.Transaction_type_id=7 THEN "points forfeited"
                   WHEN t.Transaction_type_id=30 THEN "Auction Bidding"
                  WHEN t.Transaction_type_id=30 THEN "Points Refund"
                  ELSE "-" END) AS Transaction_type'))
                ->where(function($q) {
                    return $q->where([['t.Transaction_type_id', 1], ['t.Redeem_points', '>', 0]])
                        ->orwhere([['t.Transaction_type_id', 1], ['t.Loyalty_points', '>', 0]])
                        ->orWhere([['t.Transaction_type_id', '<>', 1]]);
                })->where('t.Transaction_status',"=",0)
                 ->where("t.Membership1_id",$membership_id)->where("t.Company_id",$company_id)
                  ->orderBy("t.Transaction_id","desc");
    }

}
