<?php

namespace App\Mail\Promo;

use App\Traits\MailNotification;
use App\Traits\UserNotificationTrait;
use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Contracts\Queue\ShouldQueue;

class PromoMail extends Mailable
{
    use Queueable, SerializesModels,MailNotification,UserNotificationTrait;

    private $user;
    private $amount;
    private $notification_type;
    private $promo_code;
    private $loyalty_program;
    private $company;


    public function __construct($user,$amount,$notification_type,$promo_code,$loyalty_program,$company)
    {
        $this->user = $user;
        $this->amount = $amount;
        $this->notification_type = $notification_type;
        $this->promo_code = $promo_code;
        $this->loyalty_program = $loyalty_program;
        $this->company = $company;
    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {
        $email_structure = $this->getEmailStructure($this->company->Company_id,34);
        $email_variables = $this->getEmailVariable("Customer Promo Code");
        $values = $this->assignValues($this->user,$this->company,$this->amount,$this->promo_code);
        $email_structure->Email_subject =
            str_replace('$Promo_code', $this->promo_code,$email_structure->Email_subject);
        $email_structure->Footer_notes =
            str_replace('$Website', $this->company->Website,$email_structure->Footer_notes);
        foreach($email_variables as $variable)
        {
            if(isset($values["".$variable.""]))
                $email_structure->Email_body = str_replace($variable,$values["".$variable.""],$email_structure->Email_body);
        }
        $this->createNotification($this->user,$email_structure->Email_body,13,0);
        return $this->from($this->company->Company_person_Email,$this->company->Company_name)
            ->subject($email_structure->Email_subject)->view('mail.general')
            ->with(['structure'=>$email_structure]);
    }

    private function assignValues($user,$company,$amount,$promo_code)
    {
        return ['$First_name'=>$user->First_name,'$Last_name'=>$user->Last_name,'$Promo_code'=>$promo_code,
            '$Company_name'=>$company->Company_name,'$Membership_id'=>$user->Membership_id,'$Promo_points_gain'=>$amount,
            '$Membership_email'=>$user->User_email_id,'$Loyalty_program_id'=>$user->Loyalty_programme_id,
            '$Website'=>$company->Website,'$Application_website'=>$company->Website,'$Facebook_icon'=>'',
            '$Twitter_icon'=>'','$Google_plus_icon'=>'','$Linkedin_icon'=>'',' $iOS_icon'=>'',
            '$Android_icon'=>'','$Current_balance'=>$user->Current_balance];
    }
}
