<?php

namespace App\Models;

use Illuminate\Support\Str;
use Laravel\Passport\HasApiTokens;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;

class EnrollmentMaster extends Authenticatable
{
    use HasApiTokens,Notifiable;

    public $timestamps = false;
    protected $primaryKey="Enrollment_id";
    protected $table = "lsl_enrollment_master";

    public function getEmailForPasswordReset()
    {
       return $this->User_email_id;
    }

    public function routeNotificationFor($driver)
    {
        if (method_exists($this, $method = 'routeNotificationFor'.Str::studly($driver))) {
            return $this->{$method}();
        }

        switch ($driver) {
            case 'database':
                return $this->notifications();
            case 'mail':
                return $this->User_email_id;
            case 'nexmo':
                return $this->phone_number;
        }
    }

    public function walletBalance()
    {
        return $this->Current_balance - $this->Blocked_points;
    }

    public function format()
    {
       return (object)([
           'Last_name'=>$this->Last_name,
           "Membership_id"=>$this->Membership_id,
            'First_name'=>$this->First_name,
            'Current_balance'=>$this->Current_balance,
           'Blocked_points'=>$this->Blocked_points,
            'User_email_id'=>$this->User_email_id,
           "Phone_no"=>$this->Phone_no,
           'Total_purchase_amount'=>$this->Total_purchase_amount,
            'Total_bonus_points'=>$this->Total_bonus_points,
           'Total_redeem_points'=>$this->Total_redeem_points,
           'Total_gained_points'=>$this->Total_gained_points,
           "Tier_name"=>$this->tier->Tier_name,
           "Tier_id"=>$this->tier->Tier_id,
           "City_id"=>$this->City_id,
           "Current_address"=>$this->Current_address,
           "State_id"=>$this->State_id,
           "Country_id"=>$this->Country_id
       ]);
    }


    protected $fillable=['First_name','Last_name','Current_address','Zipcode','Country_id','Region_id',
    'Zone_id','State_id','City_id','Phone_no','Date_of_birth','Age','Sex','Married','Wedding_anniversary_date',
    'Qualification','Photograph','User_email_id','Communication_email_id','Password','Pin','Active_flag','Company_id'
    ,'Child_company_id','Referee_flag','Referee_id','Tier_id','User_group_id','User_type_id','Account_number'
    ,'Branch_code','Loyalty_programme_id',"Wallet_balance","Blocked_points"];
    protected $hidden = [
        'Password', 'Pin',
    ];

    public function program()
    {
        return $this->belongsTo('App\Models\LoyaltyProgram',
            'Loyalty_programme_id','Loyalty_program_id');
    }

    public function tier()
    {
        return $this->belongsTo('App\Models\TierMaster',
            'Tier_id','Tier_id');
    }
}
