<?php


namespace App\Traits;


use App\Jobs\CustomMailJob;
use App\Mail\Pin\AccountLockedMail;
use App\Mail\Pin\PinResetMail;
use App\Models\Company;
use App\Models\Passwords\LockedoutCustomer;
use Illuminate\Support\Str;

trait RedeemPinTrait
{

    protected function incrementRedeemPinAttempt($user,$limit)
    {
        $lockedattempt = LockedoutCustomer::firstOrNew(['Membership_id'=>$user->Membership_id,
            "Loyalty_program_id"=>$user->Loyalty_programme_id],
            ['Attempt_count'=>0,"Lockedout_flag"=>0,"Enrollment_id"=>$user->Enrollment_id]);
       $lockedattempt->Attempt_count++;
       $lockedattempt->save();
       return  $limit-$lockedattempt->Attempt_count;
    }

    protected function fireAccountLockout($user)
    {
        $lockedattempt = LockedoutCustomer::firstOrNew(['Membership_id'=>$user->Membership_id,
            "Loyalty_program_id"=>$user->Loyalty_programme_id],
            ['Attempt_count'=>3,"Lockedout_flag"=>0,"Enrollment_id"=>$user->Enrollment_id]);
        $lockedattempt->Lockedout_flag=true;
        $lockedattempt->Token = Str::random(64);
        $lockedattempt->save();

    }

    protected function clearRedeemAttempt($user)
    {
        LockedoutCustomer::where('Membership_id',$user->Membership_id)
            ->where("Loyalty_program_id",$user->Loyalty_programme_id)->delete();
    }

    protected  function hasTooManyAttempt($user,$limit):bool
    {
        $lockedattempt = LockedoutCustomer::firstOrCreate(['Membership_id'=>$user->Membership_id,
            "Loyalty_program_id"=>$user->Loyalty_programme_id],
            ['Attempt_count'=>0,"Lockedout_flag"=>0,"Enrollment_id"=>$user->Enrollment_id]);
        return $lockedattempt->Attempt_count >= $limit;
    }

    protected  function sendPinResetSuccess($user):bool
    {
        $company = Company::where("Company_id",$user->Company_id)->first();
        try{
            CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                new PinResetMail($user,$company));
            return true;
        }catch (\Exception $ex)
        {
            return false;
        }

        return false;

    }

    protected function sendPinResetRequest($user,$callback_url)
    {
        $this->clearRedeemAttempt($user);
        $lockedattempt = LockedoutCustomer::firstOrCreate(['Membership_id'=>$user->Membership_id,
            "Loyalty_program_id"=>$user->Loyalty_programme_id],
            ['Attempt_count'=>0,"Lockedout_flag"=>0,"Token"=>Str::random(64),"Enrollment_id"=>$user->Enrollment_id]);
        $company = Company::where("Company_id",$user->Company_id)->first();
        if($lockedattempt && $company)
        {
            $callback_url =$callback_url."?token=".$lockedattempt->Token;
            try
            {
                CustomMailJob::dispatch($company->mail_configuration->toArray(),$user->User_email_id,
                    new AccountLockedMail($user,$callback_url,$company));
                return true;
            }catch (\Exception $ex)
            {

                return false;
            }
        }else{
            return false;
        }
    }

    public function checkToken($user,$token)
    {
       return LockedoutCustomer::where('Membership_id',$user->Membership_id)->
        where("Loyalty_program_id",$user->Loyalty_programme_id)->where("Token",$token)->first();
    }
}
